/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */

dojo.provide("com.ibm.rational.cm.web.ui.data.CqChoiceListStore");
dojo.require("dojo.data.util.simpleFetch");
dojo.require("com.ibm.rational.cm.web._base");

(function(){

var cq = com.ibm.rational.cm.web;

dojo.declare(
	"com.ibm.rational.cm.web.ui.data.CqChoiceListStore",
	null,
	{
		url: null,
		_identifier: "name", // "name" is default searchAttr in dijit.form.ComboBox
		_loadInProgress: false,
		_loadFinished: false,
		
		constructor: function(args){
			if(args){
				dojo.mixin(this, args);
			}
			this._choiceList = [];
		},
		hasValue: function(val){
			return dojo.indexOf(this._choiceList, val) >= 0;
		},
		getIdentity: function(item){
			if(this.isItem(item)){
				return item[this._identifier];
			}
			return null;
		},
		getIdentityAttributes: function(item){
			return [this._identifier];
		},
		fetchItemByIdentity: function(keywordArgs){
			var self = this;
			var callback = function(items, keywordArgs){
				if(keywordArgs.onItem){
					if(dojo.indexOf(keywordArgs.identity, self._choiceList)){
						var item = {}; item[self._identifier] = keywordArgs.identity;
						var scope =  keywordArgs.scope?keywordArgs.scope:dojo.global;
						keywordArgs.onItem.call(scope, item);						
					} 
				}
			};
			var errorback = function(data, ioArgs){
			};
			var args = dojo.mixin(dojo.clone(keywordArgs),{start:0,count:1});
			this._fetchItems(args, callback, errorback);
		},
		getFeatures: function(){
			return {'dojo.data.api.Read': true, 'dojo.data.api.Identity':true};
		},
		getValue: function(item, attribute, defaultValue){
			if(this.isItem(item)){
				return item[attribute];
			}
			return null;
		},
		getValues: function(item, attribute){
			var value = this.getValue(item, attribute);
			return (value ? [value] : []); //Array
		},
		getAttributes: function(item){
			if(this.isItem(item)){
				return [this._identifier];
			}
			return [];
		},
		hasAttribute: function(item, attribute){
			return this.getValue(item, attribute) !== null;
		},
		containsValue: function(item, attribute, value){
			return this.getValue(item, attribute) === value;
		},
		isItem: function(something){
			if(!(something && dojo.isObject(something)))return false;
			var val = something[this._identifier];
			if(val === undefined || val === null)return false;
			return dojo.indexOf(this._choiceList, val) >= 0;
		},
		isItemLoaded: function(something) {
			return this.isItem(something);
		},
		loadItem: function(keywordArgs){
		},
		close: function(request){
			this._choiceList = []; // clear
			delete this._queuedHandlers;
		},
		getLabel: function(item){
			this.getValue(item);
		},
		getLabelAttributes: function(item){
			return [this._identifier];
		},
		_fetchItems: function(request, fetchHandler, errorHandler){
			var self = this;
			var filter = function(requestArgs, arrayOfItems, handler){
				var items = null;
				var id = self._identifier;
				var query = requestArgs.query;
				var startIndex = requestArgs.start || 0;
				var endIndex = requestArgs.count ? Math.min(arrayOfItems.length, (startIndex + requestArgs.count)) : arrayOfItems.length;
				if(query){
					var ignoreCase = requestArgs.queryOptions ? requestArgs.queryOptions.ignoreCase : false; 
					var regexpList = [];
					for(var key in query){
						var q = query[key];
						if(typeof q === "string"){
							regexpList.push({val: q, exp: dojo.data.util.filter.patternToRegExp(q, ignoreCase)});
						}
					}
					if(regexpList.length > 0 && !(regexpList.length == 1 && regexpList[0].val == "*")){
						items = [];
						var count = 0;
						for(var i = 0; i < arrayOfItems.length; ++i){
							var match = false;
							var candidateItem = arrayOfItems[i];
							if(dojo.isArray(candidateItem)) candidateItem = candidateItem[0];
							for(var j=0,l=regexpList.length; j<l; j++){
								var e = regexpList[j];
								if(e.val == candidateItem || candidateItem.match(e.exp)){
									var match = true;
								}
							}
							if(match){
								if(count >= startIndex){
									var item = {}; item[id] = candidateItem;
									items.push(item);
								}else{
									items.push(""); // dummy data for simpleFetch 
								}
								if(++count >= endIndex) break;
							}
						}
					}
				}
				if(!items) {
					items = new Array(endIndex);
					for(var i=startIndex; i<endIndex; i++){
						var item = {}; item[id] = arrayOfItems[i];
						items[i] = item;
					};
				}
				handler(items, requestArgs);
			};
			if(this._loadFinished){
				filter(request, this._choiceList, fetchHandler);
				return;
			}
			if(!this._loadInProgress){
				this._loadInProgress = true;
				cq.xhrGet({
					url: this.url,
					content: {sort: this.sort || "false"},
					load: dojo.hitch(this, function(data, ioArgs){
						var error = true;
						if(!data){
						}else if(dojo.isArray(data.choices)){
							error = false;
						}
						if(error){
							this._loadFinished = false;
							this._choiceList = [""]; // error... 							
						}else{
							this._loadFinished = true;
							this._choiceList = !this.isDate ? data.choices : 
								dojo.map(data.choices, function(v){
									dojo["require"]("dojo.date.stamp");
									var date = dojo.date.stamp.fromISOString(v);
									if(date){
										dojo["require"]("dojo.date.locale");
										return dojo.date.locale.format(date, {formatLength: "medium"});
									}
									return v;
								});
						}
						var handlers = this._queuedHandlers;
						var numHandlers = handlers ? handlers.length : 0;
						if(numHandlers > 0){
							for(var i=0; i<numHandlers; i++){
								filter(request, this._choiceList, handlers[i].fetchHandler);
							}
						}
					}),
					error: dojo.hitch(this, function(data, ioArgs){
						var handlers = this._queuedHandlers;
						var numHandlers = handlers.length;
						if(numHandlers > 0){
							for(var i=0; i<numHandlers; i++){
								handlers[i].errorHandler(data, ioArgs);
							}
						}						
					}),
					handle: dojo.hitch(this, function(data, ioArgs){
						this._loadInProgress = false;
						delete this._queuedHandlers;
					}),
					handleError: true,
					preventCache: true
				});
			}
			this._queueHandler({request:request, fetchHandler:fetchHandler, errorHandler:errorHandler});
		},
		_queueHandler: function(handlerArgs){
			if(!this._queuedHandlers) this._queuedHandlers = [];
			this._queuedHandlers.push(handlerArgs);
		}
	}
);

})();

dojo.extend(com.ibm.rational.cm.web.ui.data.CqChoiceListStore,dojo.data.util.simpleFetch);