/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllUsers");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.repository.web.client.internal.RepositoryClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("jazz.ui.FilterBox");
dojo.require("jazz.util.html");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var RepositoryClient = com.ibm.team.repository.web.client.internal.RepositoryClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var HTML = jazz.util.html;
var FilterBox = jazz.ui.FilterBox;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var PAGE_SIZE = 50;

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllUsers",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllUsers.html"),
	
	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.allUsersRelated = messages.allUsersRelated;
		this.allUsersAll = messages.allUsersAll;
		this.allDashboardsPrev = messages.allDashboardsPrev;
		this.allDashboardsNext = messages.allDashboardsNext;
		this.allDashboardsOf = messages.allDashboardsOf;
	},
	postCreate: function() {
		var filterBox = new FilterBox();
		filterBox.setChangeHandler(dojo.hitch(this,this._doFilter));
		this._filterArea.appendChild(filterBox.domNode);
		this.showPage(this.page ? this.page : "related");
		this.connect(this._previous, "onclick", this._handlePrevious);
		this.connect(this._next, "onclick", this._handleNext);
	},
	
	showPage: function(page) {
		if (page)
			this.page = page;
		if (this.page == "related")
			this._handleRelated();
		else if (this.page == "all") 
			this._handleAll();
		else {
			
			var alphaLinks = this._alphaLinks.getElementsByTagName("a");
			page = this.page == "X" ? "X-Z" : this.page;
			for (var i = 0; i < alphaLinks.length; i++) {
				if (alphaLinks[i].innerHTML == page) {
					var target = alphaLinks[i];
					break;
				}
			}
			if (target) 
				this._handleAlpha({
					target: target
				});
			else
				this.showPage("related");
		}
	},
	
	_createRow: function(text, userId) {
		var result = document.createElement("div");
		result.className = "resultRow";
		var resultHtml = [];
		result.style.paddingLeft = "1em";
		if (userId) {
			resultHtml.push("<a href='");
			resultHtml.push("#action=")
			resultHtml.push(DashboardConstants.ACTION_VIEW_DASHBOARD);
			resultHtml.push("&");
			resultHtml.push(DashboardConstants.URL_PARAM_USER);
			resultHtml.push("=");
			resultHtml.push(userId);
			resultHtml.push("'>");
		}
		resultHtml.push(HTML.escape(text));
		if (userId) {
			resultHtml.push("</a>");
		}
		result.innerHTML = resultHtml.join("");
		return result;
	},
	
	_handleRelated: function(event) {
		if (event && event.preventDefault)
			event.preventDefault();
		if (this._selectedLink == this._related)
			return;
		delete this._token;
		if (this._runQueryTimeout){
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		this._setSelectedLink(this._related);
		if (!this._relatedResults) {
			this._relatedResults = document.createElement("div");
			var serviceObject = {
				self: this,
				success: function(results){
					if (!results) {
						this.self._updateRelatedCount(0);
						this.self._relatedResults.appendChild(this.self._createRow(messages.allUsersNoRelated));
						return;
					}
					for (var i = 0; i < results.length; i++) {
						results[i].node = this.self._createRow(results[i].name, results[i].userId);
						results[i].name = results[i].name.toLowerCase();
						this.self._relatedResults.appendChild(results[i].node);
					}
					this.self._updateRelatedCount(results.length);
					this.self._relatedDataStore = results;
				},
				failure: function(error){
					dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
				}
			}
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			DashboardClient.getRelatedUsers(getCurrentProjectArea().itemId, getAuthenticatedContributor().userId, srh);
		}
		if (this._isRelatedFilterStale)
			this._doRelatedFilter();
		this._isRelatedFilterStale = false;
		this._setResultsAreaContent(this._relatedResults);
		this._pageControls.style.display = "none";
		this._relatedPageControls.style.display = "block";
	},
	
	_handleAlpha: function(event) {
		if (event && event.preventDefault)
			event.preventDefault();
		if (this._selectedLink == event.target)
			return;
		delete this._token;
		this._setSelectedLink(event.target);
		var character = event.target.innerHTML;
		this._expressions = (character == "X-Z") ? { or: ["X%", "Y%", "Z%"] } : { or: [character + "%"] };
		this._runQuery();
	},
	
	_handleAll: function(event) {
		if (event && event.preventDefault)
			event.preventDefault();
		if (this._selectedLink == this._all)
			return;
		delete this._token;
		this._setSelectedLink(this._all);
		this._expressions = { or: ["%"] };
		this._runQuery();
	},
	
	_handleNext: function(event) {
		event.preventDefault();
		if (dojo.hasClass(this._next, "disabled"))
			return;
		this._runQuery(this._startIndex + PAGE_SIZE);
	},
	
	_handlePrevious: function(event) {
		event.preventDefault();
		if (dojo.hasClass(this._previous, "disabled"))
			return;
		this._runQuery(this._startIndex - PAGE_SIZE);
	},
	
	_runQuery: function(startIndex) {
		if (this._runQueryTimeout) {
			clearTimeout(this._runQueryTimeout);
			delete this._runQueryTimeout;
		}
		if (!this._nextQueryId)
			this._nextQueryId = 1;
		if (!startIndex)
			startIndex = 0;
		var serviceObject = {
			self: this,
			id: this._nextQueryId,
			success: function(results){
				if (this.self._lastQueryId && this.self._lastQueryId > this.id)
					return;
				this.self._token = results.token;
				this.self._lastQueryId = this.id;
				this.self._renderResults(results);
			},
			failure: function(error){
				//console.error(error);
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		}
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		var args = { expression: dojo.clone(this._expressions), startIndex: startIndex, maxResults: PAGE_SIZE};
		if (this._currentFilter) {
			var filter = "%" + this._currentFilter + "%";
			args.expression = { and: [ this._expressions, filter ]};
		}
		if (this._token)
			args.token = this._token;
		RepositoryClient.getContributorQueryResults(srh, args);
		this._nextQueryId++;
	},
	
	_renderResults: function(results) {
		if (this._selectedLink == this._related)
			return;
		var div = document.createElement("div");
		if (results && results.contributors && results.contributors.length > 0) {
			for (var i = 0; i < results.contributors.length; i++) {
				div.appendChild(this._createRow(results.contributors[i].name, results.contributors[i].userId));
			}
		} else {
			div.appendChild(this._createRow(messages.allUsersNone));
		}
		this._updatePageCount(results);
		this._setResultsAreaContent(div);
	},
	
	_updatePageCount: function(results) {
		this._relatedPageControls.style.display = "none";
		if (!results || !results.contributors || !results.contributors.length) {
			dojo.addClass(this._previous, "disabled");
			dojo.addClass(this._next, "disabled");
			this._currentStartCount.innerHTML = 0;
			this._currentEndCount.innerHTML = 0;
			this._totalCount.innerHTML = 0;
		} else {
			if (results.startIndex != 0)
				dojo.removeClass(this._previous, "disabled");
			else
				dojo.addClass(this._previous, "disabled");
			if (results.totalResults - results.startIndex > PAGE_SIZE)
				dojo.removeClass(this._next, "disabled");
			else
				dojo.addClass(this._next, "disabled");
			this._startIndex = results.startIndex;
			this._currentStartCount.innerHTML = this._startIndex + 1;
			this._currentEndCount.innerHTML = Math.min(this._startIndex + PAGE_SIZE, results.totalResults);
			this._totalCount.innerHTML = results.totalResults;
		}
		this._pageControls.style.display = "block";
	},
	
	_setResultsAreaContent: function(node) {
		if (this._resultsArea.firstChild)
			this._resultsArea.replaceChild(node, this._resultsArea.firstChild);
		else 
			this._resultsArea.appendChild(node);
		var refNode = this._resultsArea.firstChild.nextSibling;
		while (refNode) {
			var nextNode = refNode.nextSibling;
			this._resultsArea.removeChild(refNode);
			refNode = nextNode;
		}
	},
	
	_setSelectedLink: function(link) {
		if (this._selectedLink)
			dojo.removeClass(this._selectedLink, "selected");
		dojo.addClass(link, "selected");
		this._selectedLink = link;
	},
	
	_doFilter: function(filter) {
		this._currentFilter = filter;
		if (this._selectedLink == this._related) 
			if (this._currentFilter.length == 0) 
				this._clearRelatedFilter();
			else
				this._doRelatedFilter();
		else {
			delete this._token;
			this._isRelatedFilterStale = true;
			// set a timeout to query with the new filter
			// need to ensure 0 is passed to _runQuery or setTimeout will pass some other value
			if (!this._runQueryTimeout) {
				this._runQueryTimeout = setTimeout(dojo.hitch(this,this._runQuery,0),500);
			}
		}
	},
	
	_doRelatedFilter: function() {
		if (!this._relatedDataStore)
			return;
		var hits = 0;
		for (var i = 0; i < this._relatedDataStore.length; i++) {
			if (this._relatedDataStore[i].name.indexOf(this._currentFilter) != -1) {
				this._relatedDataStore[i].node.style.display = "block";
				hits++;
			} else {
				this._relatedDataStore[i].node.style.display = "none";
			}
		}
		this._updateRelatedCount(hits);
		if (!hits) {
			if (!this._noRelatedHitsRow) {
				this._noRelatedHitsRow = this._createRow(messages.allUsersNoTeamMatch);
				this._relatedResults.appendChild(this._noRelatedHitsRow);
			}
			this._noRelatedHitsRow.style.display = "block";
		} else if (this._noRelatedHitsRow)
			this._noRelatedHitsRow.style.display = "none";
	},
	
	_clearRelatedFilter: function() {
		if (!this._relatedDataStore)
			return;
		this._updateRelatedCount(this._relatedDataStore.length);
		if (this._noRelatedHitsRow)
			this._noRelatedHitsRow.style.display = "none";
		for (var i = 0; i < this._relatedDataStore.length; i++) {
			this._relatedDataStore[i].node.style.display = "block";
		}
	},
	
	_updateRelatedCount: function(count) {
		var msg = (count == 1) ? messages.allUsersShowingSingular : messages.allUsersShowingPlural;
		var subst = "<span class=\"bold\">" + count + "</span>";
		this._relatedPageControls.innerHTML = dojo.string.substitute(msg, [ subst ]);
	}
	
});

})();
