/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.build.web.ui.internal.BuildQueue");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.build.web.ui.internal.utils.Table");
dojo.require("com.ibm.team.build.web.ui.internal.BuildQueryColumn");
dojo.require("com.ibm.team.build.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.build.web.client.internal.BuildClient");
dojo.require("com.ibm.team.build.web.ui.internal.utils.ErrorUtil");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.process.web.client.projectarea");

dojo.require("jazz.util.html");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.requireLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");

(function(){
	
var PlatformUI = net.jazz.ajax.ui.PlatformUI;
var Table = com.ibm.team.build.web.ui.internal.utils.Table;
var BuildClient = com.ibm.team.build.web.client.internal.BuildClient;
var ErrorUtil = com.ibm.team.build.web.ui.internal.utils.ErrorUtil;
var BuildQueryStatusColumn = com.ibm.team.build.web.ui.internal.BuildQueryStatusColumn;
var BuildQueryStateColumn = com.ibm.team.build.web.ui.internal.BuildQueryStateColumn;
var BuildQueryLabelColumn = com.ibm.team.build.web.ui.internal.BuildQueryLabelColumn;
var BuildQueryProgressColumn = com.ibm.team.build.web.ui.internal.BuildQueryProgressColumn;
var BuildQueryEstimatedTimeColumn = com.ibm.team.build.web.ui.internal.BuildQueryEstimatedTimeColumn;
var BuildQueryStartTimeColumn = com.ibm.team.build.web.ui.internal.BuildQueryStartTimeColumn;
var BuildQueryDurationColumn = com.ibm.team.build.web.ui.internal.BuildQueryDurationColumn;
var BuildQueryTagsColumn = com.ibm.team.build.web.ui.internal.BuildQueryTagsColumn;
var BuildQueryTimeInQueueColumn=com.ibm.team.build.web.ui.internal.BuildQueryTimeInQueueColumn;

var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;

var messages = dojo.i18n.getLocalization("com.ibm.team.build.web.ui.internal", "BuildMessages");
var TABLE_SPECS = {
		headings: [{label: "", width:"1%"},
				   {label: "", width:"1%"},
				   {label: messages.labelLabel, width:"20%"},
				   {label: messages.labelProgress, width:"10%"},
				   {label: messages.labelTags,width:"10%"},
				   {label: messages.labelStart, width:"22%"},
				   {label: messages.labelDuration, width:"12%"},
				   {label: messages.labelEstimated, width:"1%"},
				   {label: messages.labelTimeInQueue, width:"23%"}
				  ],
		style: com.ibm.team.build.web.ui.internal.utils.table.Styles.GOLD
	};

dojo.declare("com.ibm.team.build.web.ui.internal.BuildQueue",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.build.web", "ui/internal/templates/ListPage.html"),
	
	widgetsInTemplate: true,
	
	postCreate: function() {
		dojo.addClass(this.domNode, "BuildQuery");
		this.populateHeader();
		this._doQuery();
	},
	
	refresh: function(e) {
		if (e)
			e.preventDefault();
		PlatformUI.getWorkbench().showMessage(messages.loadingBuildQueue);
		this._refreshTime = new Date().getTime();
		this._doQuery();
	},
	
	populateHeader: function() {
		var args = {
			imgSrc: dojo.moduleUrl("com.ibm.team.build.web", "graphics/metanav/icons/refresh.gif"),
			label: messages.refresh
		};
		this.metanav.toolbar.addButton(this, dojo.hitch(this,this.refresh), args);
		var heading = dojo.string.substitute(messages.buildQueueHeading,[ getCurrentProjectArea().name]);
		this.metanav.setTitle(jazz.util.html.escape(heading));
		PlatformUI.getWorkbench().setTitle(dojo.string.substitute(messages.buildsTitle,[heading]));
	},
	
	_doQuery:function() {
		var callback = dojo.hitch(this,this._handleBuildQueueCallback);
		var args = {projectAreaItemId: getCurrentProjectArea().itemId, errback: callback, callback: callback};
		if (this._refreshTime)
			args.refreshTime = this._refreshTime;
		BuildClient.getQueue(args);
	},
	
	_handleBuildQueueCallback: function(response){
		if (!this.domNode)
			return;
		if (response instanceof Error) {
			PlatformUI.getWorkbench().hideMessageArea();
			ErrorUtil.handleErrorResponse(response);
		}
		else {
			response.sort(function(a,b) {
				return b.buildResult.buildStartTime - a.buildResult.buildStartTime;
			});
			this._removeChildren(this.content);

			if (!response || !response.length) {
				var div = document.createElement("div");
				div.className = "row none-text";
				div.innerHTML = messages.noQueueFound;
				this.content.appendChild(div);
				dojo.addClass(this.content, "bordered");
			}
			else {
				var table = new Table(TABLE_SPECS);
				var bodyContainer = table.tbody;
				if(!this._columns){ //Create column objects only once
					this._columns = [ new BuildQueryStatusColumn(),
						 			  new BuildQueryStateColumn(), 
									  new BuildQueryLabelColumn({includeDefId: true}), 
									  new BuildQueryProgressColumn(), 
									  new BuildQueryTagsColumn(),
									  new BuildQueryStartTimeColumn(),
									  new BuildQueryDurationColumn(), 
									  new BuildQueryEstimatedTimeColumn(),
									  new BuildQueryTimeInQueueColumn()
									];
				}
				for (var i = 0; i < response.length; i++) {
					this._createResultTable(bodyContainer, response[i]);
				}
				this.content.appendChild(table.domNode);
				dojo.removeClass(this.content, "bordered");
			}
			PlatformUI.getWorkbench().hideMessageArea();
		}
	},
		
	_createResultTable: function(parent, result){
		var resultRow = document.createElement("tr");
		for (var i = 0; i < this._columns.length; i++) {
		  resultRow.appendChild(this._columns[i].getCellElement(result));
		}
		parent.appendChild(resultRow);
	},
	
	_removeChildren: function(parent) {
		while (parent.firstChild)
			parent.removeChild(parent.firstChild);
	}

});
	
})();
