/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParamDateTime");

dojo.require("dojo.i18n");
dojo.require("dojo.date.locale");

dojo.require("dijit.form.DateTextBox");
dojo.require("dijit.form.TimeTextBox");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParam");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	
	var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;
	
    dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParamDateTime", [com.ibm.team.reports.web.ui.internal.ReportParam], {
        templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParamDateTime.html"),
		
		widgetsInTemplate: true,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
		},
        
		postCreate: function() {
			var self = this;
			dojo.connect(this.dateTextBox, "onChange", this, function(arg) {
				self.onDirty();
			});
			dojo.connect(this.timeTextBox, "onChange", this, function(arg) {
				self.onDirty();
			});
		},
        
		getSelectedValues: function() {
			var date = this.dateTextBox.getValue();
			if (!date || date.length == 0) {
				return null;
			}
			var time = this.timeTextBox.getValue();
			if (!time) {
				return null;
			}
			var minutes = time.getMinutes();
			var zero = "";
			if (minutes < 10) {
				zero = "0";
			}
			return [date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear() + " " +
					time.getHours() + ":" + zero + minutes];
		},
		
		setDefaultValues: function() {
			if (this.parameterDTO.defaultValue) {
				this.dateTextBox.setValue(StringUtils.stringToDate(this._getDate(this.parameterDTO.defaultValue)));
				this.timeTextBox.setDisplayedValue(this._getTime(this.parameterDTO.defaultValue));
			}
		},
		
		setValues: function(values) {
			if (values && values.length > 0) {
				this.dateTextBox.setValue(StringUtils.stringToDate(this._getDate(values)));
				this.timeTextBox.setDisplayedValue(this._getTime(values));
			}
			if (!this.isInputEnabled) {
				this.disableInput();
			}
		},
		
		remoteError: function(error) {
			alert(error.message);
		},
		
		_getDate : function(value) {
			if (!value) {
				return null;
			}
			var values =  value.split(" ");
			if (values.length < 2) {
				return value;
			}
			else {
				return values[0];
			}
		},
		
		_getTime : function(value) {
			if (!value) {
				return null;
			}
			var values =  value.split(" ");
			if (values.length < 2) {
				return value;
			}
			else {
				var result = values[1]; 
				if (values.length == 3) {
					result += values[2];
				}
				return result;
			}
		}
	});
	
})();
