/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportParamGroup");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("jazz.ui.StyledBox");

dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamFactory");

(function() {
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	dojo.declare("com.ibm.team.reports.web.ui.internal.ReportParamGroup", [dijit._Widget, dijit._Templated], {
		
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportParamGroup.html"),
		
		_execingMouseOver: false,
		_execingMouseOut: false,
			
		constructor: function(props) {
			dojo.mixin(this, props);
			this.rcpCall = false;
			this.dirtyListener = null;
			this.isObsolete = false;
		},
		
		postCreate: function() {
			this.reportParams = [];
			if (!dojo.isArray(this.parameterName)) {
				this.parameterName = [this.parameterName];
				this.parameterValue = [this.parameterValue];
			}
			if (location.href.match(/_ajax-modules/)) {
				this.rcpCall = true;
			}
			if (!this.rcpCall) {
				if (!this.navigateAwayHandle) {
					this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigate), /*priority*/ 10000);
				}
				if (!this.onBeforeUnloadHandle){
					this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this.navigateAwayEvent), /*priority*/ 10000);	
				}
			}
			var prevReportParam = null;
			for (var i = 0, len = this.parameterGroupDTO.parameters.length; i < len; i++) {
				var parameterDTO = this.parameterGroupDTO.parameters[i];
				var reportParam = com.ibm.team.reports.web.ui.internal.ReportParamFactory.create(parameterDTO, this.urlPrefix);
				reportParam.valuesMouseOver = dojo.hitch(this, this._handleValuesMouseOver);
				reportParam.valuesMouseOut = dojo.hitch(this, this._handleValuesMouseOut);
				if (prevReportParam != null) {
					// Attach child cascade to receive events
					prevReportParam.setChild(reportParam);
				} else {
					reportParam.isRoot = true;
				}
				prevReportParam = reportParam;
				
				var divToAppend = null;
				var paramClassName = "box-single";
				var appendArrow = false;
				
				if (i + 1 != len) {
					appendArrow = true;
				}
				
				if (parameterDTO.concealed) {
					paramClassName = "box-group-hidden";
				}
				
				var styledBox = new jazz.ui.StyledBox({
					closable: false, 
					primaryTitle: reportParam.getDisplayLabel() + (reportParam.isRequired() ? "*" : ""), 
					contentNode: reportParam.domNode,
					"class": paramClassName,
					trim: jazz.ui.styledBox.TrimStyles.BLUE_ALPHA_SHADOW,
					headerDefaultToHoverState: true
				});
				
				if (appendArrow && !parameterDTO.concealed) {
					var arrowDiv = document.createElement("div");
					arrowDiv.className = "cascadeArrow";
					var arrowImg = document.createElement("img");
					arrowImg.src = this.urlPrefix + "reports/resources/ui/internal/images/casc-params-arrow.png";
					arrowDiv.appendChild(arrowImg);
					divToAppend = document.createElement("div");
					divToAppend.className = "box-group-arrow";
					divToAppend.appendChild(styledBox.domNode);
					divToAppend.appendChild(arrowDiv);
				} else {
					divToAppend = styledBox.domNode;
				}
				
				this.containerDiv.appendChild(divToAppend);
				this.reportParams[parameterDTO.name] = reportParam;
				if (this.dirtyListener != null) {
					reportParam.setDirtyListener(this.dirtyListener);
				}
			}
		},
		
		destroy: function() {
			if (!this.rcpCall) {
				if (this.navigateAwayHandle) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				}
				if (this.onBeforeUnloadHandle){
					PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);	
				}
			}
		},
		
		setDirtyListener: function(object) {
			this.dirtyListener = object;
		},
		
		initGroupValues: function(reportParameterGroupDTO) {
			for (var reportParamName in this.reportParams) {
				var reportParam = this.reportParams[reportParamName];
				reportParam.init();
			}
			dojo.forEach(reportParameterGroupDTO.parameters, dojo.hitch(this, function(reportParameterDTO) {
				var recurseFunc = dojo.hitch(this, function(reportParameterDTO, isLast) {
					if (reportParameterDTO.values == undefined) {
						return;
					}
					
					var reportParam = this.reportParams[reportParameterDTO.parameterName]
					if (reportParam == null) {
						return;
					}
					
					reportParam.appendDefaultValues(reportParameterDTO, isLast);
					if (reportParameterDTO.children != undefined && reportParameterDTO.children != null) {
						for (var i = 0, len = reportParameterDTO.children.length; i < len; i++) {
							recurseFunc(reportParameterDTO.children[i], i == (len-1));
						}
					}
				});
				recurseFunc(reportParameterDTO, true);
			}));
			for (var reportParamName in this.reportParams) {
				var hasPresetParam = false;
				var reportParam = this.reportParams[reportParamName];
				for (var j = 0; j < this.parameterName.length; ++j) {
					if (this.parameterName[j] == reportParam.parameterDTO.name) {
						reportParam.setValues(this.parameterValue[j]);
						hasPresetParam = true;
						break;
					}
				}
				if (!hasPresetParam) {
					reportParam.setDefaultValues();
				}
				reportParam.postInit();
			}
		},
		
		appendReportParameters: function(reportParamsArray) {
			var appended = [];
			dojo.forEach(this.parameterGroupDTO.parameters, dojo.hitch(
				{reportParamsArray: reportParamsArray, reportParams: this.reportParams}, 
				function(parameterDTO) {
					var reportParam = this.reportParams[parameterDTO.name];
					reportParamsArray.push(reportParam);
					appended.push(reportParam);
			}));
			return appended;
		},
		
	    _handleNavigate: function(urlObj) {
			this.isObsolete = true;
			return false;
        },
        
        navigateAwayEvent: function() {
        	this.isObsolete = true;
            return false;
        },
		
		_handleValuesMouseOver: function(valuesSectionDiv) {
			if (!this._execingMouseOver) {
				this._execingMouseOver = true;
				for (var reportParamName in this.reportParams) {
					var reportParam = this.reportParams[reportParamName];
					if (reportParam._handleValuesMouseOver) {
						reportParam._handleValuesMouseOver();
					}
				}
				this._execingMouseOver = false;
			} else {
				valuesSectionDiv.style.backgroundColor = '#fbfbec';
				//dojo.addClass(valuesSectionDiv, "highlight-values");
			}
		},
		
		_handleValuesMouseOut: function(event) {
		},
		
		remoteError: function(error) {
			if (this.isObsolete) {
				return;
			}
			alert(error.message);
		}
		
	});

})();
