/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ViewletPage");

dojo.require("dojo.i18n"); 
dojo.require("dojo.string");
dojo.require("dijit.form.TextBox");

dojo.require("com.ibm.team.dashboard.web.util.internal.I18nUtil");
dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ViewletContainer");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.Observable");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("dojo.fx");
dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");


(function() {

//
// Aliases
//
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var Observable = com.ibm.team.dashboard.web.ui.internal.util.Observable;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ViewletColumn = com.ibm.team.dashboard.web.ui.internal.dnd.ViewletColumn;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletContainer = com.ibm.team.dashboard.web.ui.internal.ViewletContainer;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var I18nUtil = com.ibm.team.dashboard.web.util.internal.I18nUtil;

var messages;

/**
 * A page of viewlets (contents of one dashboard tab).
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.ViewletPage",
	[dijit._Widget, dijit._Templated, Observable], {
	
	// Variables passed into constructor
	viewletPageHelper: null,	//required
	dashboardContainer: null,	//required
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ViewletPage.html"),
	
	constructor: function(args) {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
	},
	postCreate: function() {
		this._emptyTabAddViewletLink = I18nUtil.addMessage(this._showInEditMode, dojo.string.substitute(messages.noViewletsAdd, [ messages.addViewletText ]))[0];
		this._emptyTabAddViewletEvent = dojo.connect(this._emptyTabAddViewletLink,"onclick",this.dashboardContainer,this.dashboardContainer.openViewletChooser);
		this.updateEmptyPageMessage(true);
		this.editing = (this.dashboardContainer.editMode != Constants.READ_ONLY) ? true : false;
		if (!this.editing) this._showInEditMode.style.display = "none"; 
	},
	init: function() {
		this._yieldTimeout = setTimeout(dojo.hitch(this, this.init2), 0);
		this.isActive = true;
	},
	init2: function() {
		if (!this._isDestroyed) {
			delete this._yieldTimeout;
			this._createLayout();
			this._loadingText.style.display ="none";
		}
	},
	_createLayout: function() {
		this.columns = new Array();
		this.loading = true;
		
		var content = this._tabArea;
		var columnHelpers = this.viewletPageHelper.columns;
		
		for (var i=0;i<columnHelpers.length;++i) {
			var columnWidget = this._createColumnWidget(content, {cols: columnHelpers.length, id: "column" + (i+1), columnIndex: i, columnHelper: columnHelpers[i]});
			columnWidget.addListener(dojo.hitch(this, this._onChange), Constants.PROPERTY_DASHBOARD);
			this.columns.push(columnWidget);
			this._populateColumnViewlets(columnWidget, columnHelpers[i]);
		}
		this.updateEmptyPageMessage();
		this.loading = false;
	},
	_createColumnWidget: function(contentArea, params) {
		var column = document.createElement("div");
		if (params.cols == 1) {
			column.className = "one-col";
		} else if (params.cols == 2) {
			column.className = "two-col";
		} else {
			column.className = "three-col";
		}
		column.id = params.id;
		contentArea.appendChild(column);
		return new ViewletColumn(column, {accept: ["viewlet"], columnIndex: params.columnIndex, columnHelper: params.columnHelper, viewletPage: this, editMode: this.editing});
	},
	isVisible: function() {
		return (this.dashboardContainer.currentPage == this) && this.dashboardContainer.isVisible();
	},	
	setEditMode: function(editing) {
		this.editing = editing;
		for (var i = 0; i < this.columns.length; i++) {
			this.columns[i].setEditMode(editing);
		}
		if (editing) {
			this._showInEditMode.style.display = "";
		} else {
			this._showInEditMode.style.display = "none";
		}
	},
	setActive: function(/* boolean */ active) {
		this.isActive = active;
		for (var j = 0; this.columns && j < this.columns.length; j++) {
			var widgets = this.columns[j].widgets;
			for (var k = 0; widgets && k < widgets.length; k++) {
				var widget = widgets[k];
				if (widget && widget._viewlet) {
					widget.setActive(active);
					if (widget._settingsOpen && widget._settingsWidget)
						widget._settingsWidget._resize();
				}
			} 	
		}
	},
	_populateColumnViewlets: function(columnWidget, /* Column */ columnHelper) {
		// add viewlets
		var columnIndex = columnWidget.index;
		var viewlets = columnHelper.viewlets;
		if (viewlets) {
			for (var i = 0; i < viewlets.length; i++) {
				this._addViewlet(viewlets[i], columnIndex, -1);
			}
		}
	},
	/*
	 * Adds a copy of the viewlet represented by the provided viewlet helper.
	 */
	addViewletFromHelper: function(/* string */ viewletHelper, columnIndex, viewletIndex) {
		this.addViewlet(viewletHelper.title, {id: viewletHelper.definitionId, version: viewletHelper.version}, viewletHelper.icon, viewletHelper.preferences, viewletHelper.scope, viewletHelper.scopeItem ? viewletHelper.scopeItem.itemId : null, viewletHelper.scopeSubteams? true : false, viewletHelper.showBackground ? true : false, viewletHelper.viewletTrim, viewletHelper.memento, columnIndex || 0, viewletIndex || 0, viewletHelper.userTitle ? true : false, viewletHelper.origin);
	},
	/*
	 * Adds a new viewlet to the dashboard, assigns it a new ID (fetched
	 * from the server), and updates the server state, asynchronously.
	 */
	addViewlet: function(/* string */ title, /* object */ definition, /* string */ icon, /* object */ preferences, /* number */ scope, /* string */ scopeItemId, /* boolean */ scopeSubteams, /* boolean */ showBackground, /* number */ trim, /* string */ memento, /* number */ destColumnIndex, /* number */ destViewletIndex, /* boolean */ userTitle, /* string */ origin, /* string */ uuid) {
		// first get a UUID
		var serviceObject = {
			self: this,
			success: function(uuids) {
				if (this.self._isDestroyed) return;
				// add the viewlet at top of first column
				var viewletHelper = {
					id: uuids[0],
					title: title,
					definitionId: definition.id,
					collapsed: false,
					scopeSubteams: scopeSubteams,
					showBackground: showBackground,
					userTitle: userTitle
				};
				// optional fields; only add if present
				if (icon) {
					viewletHelper.icon = icon;
				}
				if (memento) {
					viewletHelper.memento = memento;
				}
				if (preferences) {
					viewletHelper.preferences = preferences.length ? preferences : Util.toEntryArray(preferences);
				}
				if (scope) {
					viewletHelper.scope = scope;
				}
				if (scopeItemId) {
					viewletHelper.scopeItem = {itemId: scopeItemId };
				}
				if (trim) {
					viewletHelper.viewletTrim = trim;
				}
				if (definition.version) {
					viewletHelper.version = definition.version;
				}
				if (origin) {
					viewletHelper.origin = origin;
				}

				// add to data model
				var columnHelper = this.self.viewletPageHelper.columns[destColumnIndex];
				var viewlets = columnHelper.viewlets;
				if (viewlets) {
					viewlets.splice(destViewletIndex, 0, viewletHelper);
				}
				else {
					columnHelper.viewlets = [viewletHelper];
				}
				this.self._addViewlet(viewletHelper, destColumnIndex, destViewletIndex);
				this.self.propertyChanged(Constants.PROPERTY_DASHBOARD);
			},
			failure: function(errorObject) {
				errorObject.message = messages.addViewletRemoteError + errorObject.message;
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [errorObject]);
			}
		};
		if (uuid) {
			serviceObject.success([uuid]);
		} else {
			var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
			DashboardClient.generateUUID(1, srh);
		}
	},
	
	/*
	 * Adds a viewlet widget to the dashboard either at the top or bottom
	 * of a specific column (does not update server state). A viewlet index
	 * of -1 or undefined will place the viewlet at the bottom of the
	 * column.
	 */
	_addViewlet: function(/* ViewletHelper */ viewletHelper, /* number */ columnIndex, /* number */ viewletIndex) {
		// set up params
		var args = {
			dashboardBundle: this.dashboardContainer.dashboardBundle,
			viewletHelper: viewletHelper,
			viewletPage: this,
			_columnIndex: columnIndex,
			editing: this.editing
		};
		// add viewlet to destination column
		var column = this.columns[columnIndex];
		try {
			var viewletContainer = new ViewletContainer(args);	
			viewletContainer.addListener(dojo.hitch(this, this._onChange), Constants.PROPERTY_DASHBOARD);
			viewletContainer.addListener(dojo.hitch(this, this._onChange), Constants.PROPERTY_CONTRIBUTOR_INFO);
			
			if (viewletIndex != -1 && column.widgets.length >= viewletIndex) {
				// insert at given index
				column.add(viewletContainer, {index: viewletIndex, handle: viewletContainer._header, type: "viewlet", animate: true});
			} else {
				// append to the end of the column
				column.add(viewletContainer, {handle: viewletContainer._header, type: "viewlet", animate: false});
			}
			this._setEmptyTabMessage(false);
		} catch(e) {
			console.error(e);
		}
	},

	updateEmptyPageMessage: function(isLoading) {
		// check if there are any viewlets on this viewlet page
		var isEmpty = true;
		var cols = this.viewletPageHelper.columns;
		for (var i=0;i<cols.length;i++) {
			var col = cols[i];
			if(col.viewlets && col.viewlets.length >= 1) {
				isEmpty = false;
				break;
			}
		}
		if (isLoading && isEmpty) this._loadingText.style.display = "none";
		this._setEmptyTabMessage(isEmpty);
	},
		
	_setEmptyTabMessage: function(isVisible) {
		if (isVisible) {
			this._emptyTabMessage.style.display = "block";	
		} else {
			this._emptyTabMessage.style.display = "none";
		}	
	},
	/*
	 * Destroys all viewlet widgets so that they can be garbage collected.
	 */
	destroy: function() {
		this._isDestroyed = true;
		// are we in the middle of loading?
		if (this._yieldTimeout) {
			// cancel loading
			clearTimeout(this._yieldTimeout);
			delete this._yieldTimeout;
		}
		while(this.columns && this.columns.length > 0) {
			this.columns.pop().destroy();
		}
		if (this._emptyTabAddViewletEvent) dojo.disconnect(this._emptyTabAddViewletEvent);
		this.columns = null;
		this.inherited(arguments);
	},
	refreshPage: function() {
		this.dashboardContainer._refreshCurrentTab();
	},
	refreshViewlets: function() {
		for (var i=0;i<this.columns.length;i++) {
			var viewlets = this.columns[i].widgets;
			for (var j=0;j<viewlets.length;j++) {
				viewlets[j].refreshViewlet();
			}
		}
	},
	/*
	 * Utility empty callback for success responses we don't need to
	 * react to.
	 */
	_empty: function() {
	},	
	/*
	 * Utility method to log remote errors when calling services.
	 */	
	_logError: function(errorObj) {
		console.error("DashboardContainer.js remote error: " + errorObj.message);
	},
	_onChange: function(/* string */ property) {
		// notify listeners that the content has changed
		this.propertyChanged(property);
	}
});
})();
