﻿/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
class HorizontalBarGrid extends Grid {

	public static var LABEL_MARGIN:Number = 6;	
	
	public static var MIN_V_TICK_SPACING:Number = 16;
	
	
	public function HorizontalBarGrid(width:Number, height:Number, startX:Number, startY:Number) {
		super();
		
		this._calculatePartitions(width, 
								  height, 
								  startX, 
								  startY);
		
	}	

	private function _calculatePartitions(width:Number, height:Number, startX:Number, startY:Number):Void {

		var model:ChartModel = ChartEngine.getInstance().getModel();		
		var graphAreaHeight:Number = height - 
									 Axis.TICK_LENGTH -
									 XAxis.LABEL_FORMAT.getTextExtent("0").height;

		// TODO: Set limit on section count
		var yAxisSectionCount:Number = model.parameterCount;
		var yAxisSpacing:Number = graphAreaHeight / yAxisSectionCount;

		var yAxisMaxLabelWidth:Number = model.getMaxParameterNameWidth(YAxis.LABEL_FORMAT);
		var maxYAxisTextFieldWidth:Number = Math.floor(Stage.width * HorizontalBarYAxis.MAX_TEXT_FIELD_WIDTH_RATIO);
		if (yAxisMaxLabelWidth > maxYAxisTextFieldWidth) {
			yAxisMaxLabelWidth = maxYAxisTextFieldWidth;
		}				
		
		var xAxisMaxLabelWidth:Number = XAxis.LABEL_FORMAT.getTextExtent(String(model.maxValue)).width;
		var graphAreaWidth:Number = width - 
									yAxisMaxLabelWidth - 
									HorizontalBarGrid.LABEL_MARGIN - 
									Math.round(xAxisMaxLabelWidth / 2);  // Last label centered under last tick (end of X-Axis)
		
		var minSpacing:Number = xAxisMaxLabelWidth + (Constants.TEXT_FIELD_MARGIN * 4);
		if (minSpacing < HorizontalBarGrid.MIN_V_TICK_SPACING) {
			minSpacing = HorizontalBarGrid.MIN_V_TICK_SPACING;
		}
		
		var xAxisSectionCount:Number = Math.floor(graphAreaWidth / minSpacing);
		
		// Calculate the maximum number of sections available (assume tightest tick spacing) given the available width
		var result:Object = this.calculateIncrement(xAxisSectionCount, 
													model.maxValue, 
													graphAreaWidth, 
													minSpacing);

		// The max value may have been increased.
		// Recompute the max label width based on the new value
		// Note: The pre-computed max value may erroneously affect the graph area width calculation
		xAxisMaxLabelWidth = XAxis.LABEL_FORMAT.getTextExtent(String(result.maxValue)).width;
		
		// Set data
		this.initializeGridData(width, 
								height, 
								result.maxValue, 
								startX, 
								startY,
								xAxisMaxLabelWidth,
								yAxisMaxLabelWidth,
								graphAreaWidth, 
								graphAreaHeight, 
								yAxisSpacing, 			// Y-Axis
								0, 						// Y-Axis: Not used
								yAxisSectionCount, 		// Y-Axis
								result.spacing, 		// X-Axis
								result.increment, 		// X-Axis
								result.sectionCount); 	// X-Axis
		
		
	}
	
	public function initializeGridData(width:Number, height:Number, max:Number, startX:Number, startY:Number, xAxisMaxLabelWidth:Number,
									   yAxisMaxLabelWidth:Number, graphAreaWidth:Number, graphAreaHeight:Number, 
									   yAxisSpacing:Number, yAxisIncrement:Number, yAxisSectionCount:Number, xAxisSpacing:Number, 
									   xAxisIncrement:Number, xAxisSectionCount:Number):Void {

		super.initializeGridData(width, height, max, startX, startY, xAxisMaxLabelWidth, yAxisMaxLabelWidth, graphAreaWidth, 
			  					 graphAreaHeight, yAxisSpacing, yAxisIncrement, yAxisSectionCount, xAxisSpacing, 
 		      					 xAxisIncrement, xAxisSectionCount);
		
		// TODO: REVISIT: Should h tick offset vars be in grid data?
		this.gridData.hTickOffsetX = 0; // Not applicable
		this.gridData.hTickOffsetY = 0; // Not applicable
		
		this.gridData.xZeroPosition = startX + yAxisMaxLabelWidth + HorizontalBarGrid.LABEL_MARGIN;		
		this.gridData.yZeroPosition = startY + graphAreaHeight;
		
		// TEST
		//Tester.dumpGridData(this.gridData);
	}	
	
	public function draw(parentMC:MovieClip):Void {
		super.draw(parentMC);
		
		// X-Axis line
		this.xAxis = new HorizontalBarXAxis();
		this.xAxis.draw(this.mc, 
						this.gridData);

		// Y-Axis line
		this.yAxis = new HorizontalBarYAxis();
		this.yAxis.draw(this.mc, 
						this.gridData);
		
	}		
	
	public function destroy():Void {
		super.destroy();
	}	
	
}
