/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
// declare the Dojo module
 dojo.provide("com.ibm.team.workitem.web.ui.internal.module.ModuleTestWidget");

// declare dependencies on other Dojo modules
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.ComboBox");
dojo.require("dojox.data.dom");

dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.app.events");
dojo.require("jazz.app.proxy");
dojo.require("jazz.util.html");
dojo.require("jazz.client.xhr");
dojo.require("com.ibm.team.workitem.web.ui2.internal.dialog.PolicyDialog");
dojo.require("com.ibm.team.workitem.web.cache.internal.PolicyRequestProxy"); 

dojo.require("jazz.app.ResourceLink");

dojo.require("dojox.xml.DomParser");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "ModuleMessages");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.parts.RemoteItemPicker");

(function() {

var RemoteItemPicker = com.ibm.team.workitem.web.ui.internal.view.editor.parts.RemoteItemPicker;
var DomParser= dojox.xml.DomParser;

var ServiceResponseHandler= com.ibm.team.repository.web.transport.ServiceResponseHandler;

// declare and implement the widget "class"
dojo.declare("com.ibm.team.workitem.web.ui.internal.module.ModuleTestWidget", [ dijit._Widget, dijit._Templated ], {

URL_ITEMS: null,
serverUrlCombo: null,

templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/module/templates/ModuleTestWidget.html"),

constructor: function(params) {
	this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "ModuleMessages");
	
},

postCreate: function() {
	this.serverUrlInput.value= "https://" + location.host + net.jazz.ajax._contextRoot;
	

	dojo.addOnLoad( dojo.hitch(this,  this._updateProjectAreaCombo));
},


_updateProjectAreaCombo: function(e) {
    var documentURL= this.serverUrlInput.value +  '/process/project-areas';
    var processCatalog= dojo.hitch(this, this._fillProjectAreaCombo);

    this.loadDocument(documentURL, processCatalog);
},

_fillProjectAreaCombo: function(projectAreaDescXML) {
    var jsdom = DomParser.parse(projectAreaDescXML);
    var nodes= this._getElementByTagName(jsdom, "jp06:project-areas");
    
    var select= document.createElement("select");
    select.className="textinput";
    dojo.connect(select, 'onchange', dojo.hitch(this, this.onProjectAreaChange));
    
    for (var i= 0; i < nodes.length; i++) {
       var projectAreas= this._getElementByTagName(nodes[i], "jp06:project-area");
        for (var k= 0; k < projectAreas.length; k++) {
	        var res= this._getAttribute(projectAreas[k], "jp06:name");
	        
	        var option = document.createElement("option");
	        option.value= res;
			option.appendChild(document.createTextNode(res));
	        select.appendChild(option);
	    }
	}
	
	if (this.projectAreaInput) {
		this.projectAreaInputContainer.removeChild(this.projectAreaInput);
	}
	this.projectAreaInput= select;
	
	this.projectAreaInputContainer.appendChild(select);
	
	this.onProjectAreaChange();
},

onRefreshButton: function(e) {
    this._updateProjectAreaCombo();
},


onPickerButton: function(e) {
    var dialogDesc= {
           name: "Selection dialog",
           path: [ "oslc_cm:ServiceDescriptor", "oslc_cm:changeRequests", "oslc_cm:selectionDialog[@oslc_cm:default='true']"]
    };
    
    var processServiceDocument= dojo.hitch(this, this.processServiceDocument, dialogDesc);
    
    if (this.requestMethod.value === "local") {
        this.loadServiceDocumentFromCatalog(this.projectAreaInput.value, processServiceDocument);
    } else {
        this.loadServiceDocumentFromProjectLink(this.projectAreaInput.value, processServiceDocument);
    }
 },
onCreateButton: function(e) {
    var dialogDesc= {
      name: "Creation dialog",
      path: [ "oslc_cm:ServiceDescriptor", "oslc_cm:changeRequests", "oslc_cm:creationDialog[@oslc_cm:default='true']"]
    };
    if (this.initOnCreateCheckBox.checked) {
        dialogDesc.extraArguments="dc%3Aname=TestName&dc%3Adescription=TestDescription%0D%0ASecond";
    }
    
    var processServiceDocument= dojo.hitch(this, this.processServiceDocument, dialogDesc);
    
    if (this.requestMethod.value === "local") {
        this.loadServiceDocumentFromCatalog(this.projectAreaInput.value, processServiceDocument);
    } else {
        this.loadServiceDocumentFromProjectLink(this.projectAreaInput.value, processServiceDocument);
    }
},

onSaveDialogButton: function(e) {

	var messageDTOs = [
		{
			message: "Test problem message",
			details: "Details of the message"
		},
		{
			message: "Another problem",
			details: "more details"
		}
	];
   
	var presentations = [
		{
			widget: "com.ibm.team.workitem.web.ui.internal.view.layout.IconMessageWidget",
			message: this.messages.CALMBackLinkProblemDialog_DIALOG_DESCRIPTION,
			severity: 4, // info
			iconSize: 1 // big	
		},
		{
			widget: "com.ibm.team.workitem.web.ui2.internal.widget.DialogListWidget",
			attributeId: "messageDTOs",
			messageSelector: "message",
			detailsSelector: "details",
			size: 4
		},
		{
			widget: "com.ibm.team.workitem.web.ui.internal.view.editor.parts.RadioGroupPart",
			attributeId: "saveOption",
			attributeValue: "1",
			possibleValues: [ 
				{
					label: this.messages.CALMBackLinkProblemDialog_RETRY_LABEL,
					id: "1"
				},
				{
					label: this.messages.CALMBackLinkProblemDialog_REVERT_LABEL,
					id: "2"
				},				
				{
					label: this.messages.CALMBackLinkProblemDialog_SKIP_BACK_LINKS_LABEL,
					id: "3"
				}
				]
					
		},
		{
			widget: "com.ibm.team.workitem.web.ui.internal.view.layout.IconMessageWidget",
			message: this.messages.CALMBackLinkProblemDialog_SKIP_BACK_LINKS_INFO,
			severity: 4, // info
			iconSize: 0 // small	
		}
		
	];
	
	var policyRequest = {
		attributes: {
			messageDTOs: messageDTOs,
			saveOption: {id: "1"}
		}
	}
	var policyRequestProxy =  new com.ibm.team.workitem.web.cache.internal.PolicyRequestProxy({ object: policyRequest}); 
	
	var parms = {
		dialogTitle: this.messages.CALMBackLinkProblemDialog_BUTTON_SAVE,
		secondaryTitle: "",
		workingCopy: policyRequestProxy,
		OkLabel: this.messages.CALMBackLinkProblemDialog_BUTTON_SAVE,
		onOk: this._handleDialogOk,
		onCancel: this._handleDialogCancel,
		presentations: presentations
	}
	new com.ibm.team.workitem.web.ui2.internal.dialog.PolicyDialog(parms);
},
_handleDialogOk: function(workingCopy) {
	console.debug("OK saveOption=" + workingCopy.getValue({ path: "attributes.saveOption"}).id );
},
_handleDialogCancel: function() {
	console.debug("Cancel");
},
onProjectAreaChange: function(e) {
    this.log("Changed to " + this.projectAreaInput.value);
},

// load service document

loadServiceDocumentFromCatalog: function(projectAreaName, processServiceDocument) {
	var documentURL= this.serverUrlInput.value + '/oslc/workitems/catalog.xml';
    var processCatalog= dojo.hitch(this, this.processCatalog, projectAreaName, processServiceDocument);
    this.loadDocument(documentURL, processCatalog);
},


loadServiceDocumentFromProjectLink: function(projectAreaName, processServiceDocument) {
	var documentURL= this.serverUrlInput.value +  '/process/project-areas';
    var processCatalog= dojo.hitch(this, this.processProjectInfo, projectAreaName, processServiceDocument);
    this.loadDocument(documentURL, processCatalog);
},


processProjectInfo: function(projectAreaName, processServiceDocument, projectAreaDescXML) {
    var jsdom = DomParser.parse(projectAreaDescXML);
    var linkDescriptionUrl= this.getNodeText(jsdom, [ "project-areas", "project-area[@name='" + projectAreaName + "']", "links-url"]);
    if (linkDescriptionUrl) {
        var processProjectLinks= dojo.hitch(this, this.processProjectLinks, projectAreaName, processServiceDocument);
        this.loadDocument(linkDescriptionUrl, processProjectLinks);
    } else {
        this.log("Links URL for " + projectAreaName + " not found. " + projectAreaDescXML);
    }
},

processProjectLinks: function(projectAreaName, processServiceDocument, projectLinkDocXML) {
    var serviceURL= this.findServiceURLInProjectLinkDoc(projectLinkDocXML);
    if (serviceURL) {
        this.log("Service URL for " + projectAreaName + ": " + serviceURL);
        
        this.loadDocument(serviceURL, processServiceDocument);
    } else {
        this.log("Service URL for " + projectAreaName + " not found. " + projectLinkDocXML);
    }
},

findServiceURLInProjectLinkDoc : function(projectLinkDocXML) {
    var jsdom = DomParser.parse(projectLinkDocXML);
    var linkName= "tests";
    var serviceNode= this.getNode(jsdom, [ "links", "jp:link[jp:link-type='" + linkName + "']", "oslc_disc:ServiceProvider", "oslc_disc:services"]);
    if (serviceNode) {
        return serviceNode.getAttribute("rdf:resource");
    }
    return null;
},

loadDocument: function(documentURL, onSuccess) {	
	this.log('Loading ' + documentURL + "...");

	 // Make an HTTP request on a secured resource.
    var deferred = jazz.client.xhrGet({
        url: documentURL,
        headers: {}
    });

    deferred.addCallback(function(result) {
    	onSuccess(result);
        return result; // for next in deferred chain
    });

    deferred.addErrback(dojo.hitch(this, this.handleError, documentURL));
},

processCatalog: function(projectAreaName, processServiceDocument, catalogXML) {
    this.log('Document loaded.');

    var serviceURL= this.findServiceURLInCatalog(catalogXML, projectAreaName);
    if (serviceURL) {
        this.log("Service URL for " + projectAreaName + ": " + serviceURL);
        this.loadDocument(serviceURL, processServiceDocument);
    } else {
        this.log("Service URL for " + projectAreaName + " not found. " + catalogXML);
    }

	//alert('catalog: ' + e);
},

findServiceURLInCatalog : function(catalogXML, projectAreaName) {
    var jsdom = DomParser.parse(catalogXML);
    var res= this.getNode(jsdom, [ "oslc_disc:ServiceProviderCatalog", "oslc_disc:entry", "oslc_disc:ServiceProvider[dc:title='" + projectAreaName + "']" , "oslc_disc:services"]);
    if (res) {
        return res.getAttribute("rdf:resource");
    }
    return null;
},

processServiceDocument: function(dialogDescription, serviceXML) {
    var dialogInfo= this.findDialogInfo(dialogDescription, serviceXML);
    if (dialogInfo) {
        if (dialogDescription.extraArguments) {
            var idx= dialogInfo.url.indexOf('?');
            if (idx != -1) {
                dialogInfo.url += '&' + dialogDescription.extraArguments;
            } else {
                dialogInfo.url += '?' + dialogDescription.extraArguments;
            }
        }
        
        this.log(dialogDescription.name + ':');
        this.log("Title: " + dialogInfo.title);
        this.log("URL: " + dialogInfo.url);
        this.log("Height: " + dialogInfo.heightHint);
        this.log("Width: " + dialogInfo.widthHint);
        
        this.openDialog(dialogInfo);
    }
},

findDialogInfo : function(dialogDescription, serviceXML) {
	var jsdom = DomParser.parse(serviceXML);
    var node= this.getNode(jsdom, dialogDescription.path);
    if (node) {
        return {
            title: this.getNodeText(node, ["dc:title"]),
            url: this.getNodeText(node, ["oslc_cm:url"]),
            widthHint: this._getAttribute(node, "oslc_cm:hintWidth"),
            heightHint: this._getAttribute(node, "oslc_cm:hintHeight"),
            namespace: "oslc_cm"
        };
    }
    return null;
},

openDialog: function(dialogDescription) {
    var self= this;

	var url= dialogDescription.url;
	if (dojo.config.isDebug) {
		var idx= url.indexOf('?');
		if (idx != -1) {
		    url= url.substring(0, idx + 1) + 'debug=true&' + url.substring(idx + 1);
		} else {
		    url= url + '?debug=true';
		}
	}
	
	var params= {
            dialogTitle: dialogDescription.title,
            secondaryTitle: "",
            width: dialogDescription.widthHint,
            height: dialogDescription.heightHint,
            namespace: dialogDescription.namespace,
            parentElement: this.domNode,
            onOk: function(result) {
	            self.addLinks(result);
                self.logResult(result);
            },
            onClose: function() {
            },
            pickerURL: url
        };	            
    new RemoteItemPicker(params);    
},

addLinks: function(results) {

	var doc= dojo.doc;

    for (var i= 0; i < results.length; i++) {
        var item= results[i];
        
        var tr = doc.createElement("tr");
        this.linksTable.appendChild(tr);
        var td = doc.createElement("td");
        tr.appendChild(td);
        
        var linkUri= item.url;
        var linkName= item.label;
        
        var anchor=doc.createElement("a");
        anchor.appendChild(doc.createTextNode(linkName));
        anchor.href = linkUri;
        anchor.target = "_blank";
   
        new jazz.app.ResourceLink({ uri: linkUri, label: jazz.util.html.escape(linkName) }, anchor);
   
    	td.appendChild(anchor);

		// add link button
        
        var td = doc.createElement("td");
        tr.appendChild(td);
        var button=doc.createElement("button");
        button.appendChild(doc.createTextNode("Add Link"));
        td.appendChild(button);
        
        // add link input
        
		var td = doc.createElement("td");
        tr.appendChild(td);
        var linkTargetInput=doc.createElement("input");
        linkTargetInput.className= "textinput";
        td.appendChild(linkTargetInput);
        
        // add link type combo
        
       	var td = doc.createElement("td");
        tr.appendChild(td);
		var linkTypeSelect= doc.createElement("select");
        td.appendChild(linkTypeSelect);
        
        var linkTypes= [ 'oslc_cm:relatedChangeManagement', 'calm:testedByTestCase', 'calm:affectsExecutionResult', 'calm:blocksTestExecutionRecord', 'calm:implementsRequirement' ];
        for (var i= 0; i < linkTypes.length; i++) {
			var option= doc.createElement("option");
        	linkTypeSelect.appendChild(option);
        	option.value= linkTypes[i];
        	option.appendChild(doc.createTextNode(linkTypes[i]));
        }        
        dojo.connect(button, "click", dojo.hitch(this, this.onAddLink, linkUri, linkTargetInput, linkTypeSelect));
    }
},

onAddLink: function(link, linkTargetInput, linkTypeSelect) {
    var self= this;
    var linkTarget= linkTargetInput.value;
    var linkType= linkTypeSelect.value;
    
    
    var itemURL= link + '?oslc_cm.properties=' + escape(linkType);
    
    var deferredGet = jazz.client.xhrGet({
        url: itemURL,
        handleAs: "json",
        headers: {
        	"Accept": "application/json"
        }
    });
    deferredGet.addCallback(function(result) {
        self.log("Get result is: " + dojo.toJson(result));
        
        var etag= deferredGet.ioArgs.xhr.getResponseHeader('Etag');
        
        var newLink= {};
        newLink["oslc_cm:label"]= linkTarget;
        newLink["rdf:resource"]= linkTarget;
        
        var newLinkCollection= result[linkType] || [ ]; // keep existing links
        newLinkCollection.push(newLink);
           
        var data= {};
        data[linkType]= newLinkCollection;
            
    	var deferredPut = jazz.client.xhrPut({
        	url: link,
        	postData: dojo.toJson(data),
			headers: {
        		"Accept": "application/json",
        		"If-Match": etag
			},
        	contentType: "application/x-oslc-cm-change-request+json" 
    	});
    	deferredPut.addCallback(function(result) {
    		self.log("Link to itself added.");
    	});
    	deferredPut.addErrback(dojo.hitch(this, this.handleError, link));
    	
        return result; // for next in deferred chain
    });
    deferredGet.addErrback(dojo.hitch(this, this.handleError, itemURL));    
},


logResult: function(result) {
   this.log(dojo.toJson(result));
},


// -------------- XML queries -----------------------

getNodeText: function(jsdom, segments) {
    var node= this.getNode(jsdom, segments);
    if (node) 
        return this._evalNodeText(node);
    return null;
},

NAMESPACES: {
    "oslc_cm": "http://open-services.net/xmlns/cm/1.0/", 
    "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#", 
    "calm": "http://jazz.net/xmlns/prod/jazz/calm/1.0/", 
    "dc": "http://purl.org/dc/terms/", 
    "oslc_disc": "http://open-services.net/xmlns/discovery/1.0/",
    "jp": "http://jazz.net/xmlns/prod/jazz/process/1.0/",
    "jp06": "http://jazz.net/xmlns/prod/jazz/process/0.6/"
},

getNode: function(xmlNode, pathSegments) {
    if (pathSegments.length == 0) {
        return xmlNode;
    }
    
    var first= pathSegments[0];
    var segments= pathSegments.slice(1); // make a copy
    
    var m1= first.match(/([\d\w-_:]+)\[(@?[\d\w-_:]+)='([^']+)'\]/);   // 0[1='2']
    if (!m1) {
        // simple element name
        var nodes= this._getElementByTagName(xmlNode, first);
        for (var i= 0; i < nodes.length; i++) {
            var res= this.getNode(nodes[i], segments);
            if (res)
                return res;
        }
        return null;
    }
    var elementName= m1[1];
    var matchName= m1[2];
    var matchVal= m1[3];
    
    var isAttribMatch= matchName.charAt(0) == '@';
    if (isAttribMatch)
       matchName= matchName.substring(1);


    var nodes= this._getElementByTagName(xmlNode, elementName);
    for (var i= 0; i < nodes.length; i++) {
       var node= nodes[i];
       if (isAttribMatch) {
           if (this._getAttribute(node, matchName) == matchVal)
               return this.getNode(node, segments);
       } else {
           var innerNodes= this._getElementByTagName(node, matchName);
           if (innerNodes.length == 1 && this._evalNodeText(innerNodes[0]) == matchVal)
               return this.getNode(node, segments);
       }
    }
    return null;
},
_getElementByTagName: function(node, name) {
    var s= name.split(':', 2);
    if (s.length == 1) {
        return node.getElementsByTagName(name);
    } else {
        var ns= this.NAMESPACES[s[0]];
        return node.getElementsByTagNameNS(s[1], ns);
    }
},

_getAttribute: function(node, name) {
    var s= name.split(':', 2);
    if (s.length == 1) {
        return node.getAttribute(name);
    } else {
        var ns= this.NAMESPACES[s[0]];
        return node.getAttributeNS(s[1], ns);
    }
},

_evalNodeText: function(node) {
    if (node.childNodes.length == 1 && node.childNodes[0].nodeName == '#text') {
         return node.childNodes[0].nodeValue;
    }
    return null;
},

handleError: function(errorObj, documentURL) {
    // This will get called either if the user failed to authenticate and canceled the
    // dialog or if authentication succeeded but the request had some other problem.
    if(errorObj.dojoType === "cancel") {
        alert("User cancelled request.");
    } else {
        alert("An error occurred while accessing " +  documentURL + ": "+ errorObj);
    }
    return errorObj;
},

log: function(text) {
	this.logText.value += text + '\n';

},

// ------------------------------------

onQueryButton: function(e) {    
    var processServiceDocument= dojo.hitch(this, this.processServiceDocumentForQuery, this.queryInput.value);
    
	if (this.requestMethod.value === "local") {
        this.loadServiceDocumentFromCatalog(this.projectAreaInput.value, processServiceDocument);
    } else {
        this.loadServiceDocumentFromProjectLink(this.projectAreaInput.value, processServiceDocument);
    }
},

processServiceDocumentForQuery: function(query, serviceXML) {
	var self= this;

	var path= [ "oslc_cm:ServiceDescriptor", "oslc_cm:changeRequests", "oslc_cm:simpleQuery", "oslc_cm:url"];
	var jsdom = DomParser.parse(serviceXML);
    var queryBaseURL= this.getNodeText(jsdom, path);
    if (queryBaseURL) {
    	this.log("Query Base URL: " + queryBaseURL);
    	
    	var queryURL= queryBaseURL + '?';
    	if (query.length > 0) {
    	    queryURL += 'oslc_cm.query=' + escape(query) + '&';
    	}
		queryURL += 'oslc_cm.properties=' + escape('dc:title,rdf:resource,dc:modified');

	 	// Make an HTTP request on a secured resource.
	    var deferred = jazz.client.xhrGet({
	        url: queryURL,
			headers: {
				accept: "text/json"
			}
	    });
	
	    deferred.addCallback(function(result) {
	    	self.queryLoaded(result);
	        return result; // for next in deferred chain
	    });
	
	    deferred.addErrback(dojo.hitch(this, this.handleError, queryURL));

    	
    }
},

queryLoaded: function(queryResult) {
     this.log("Query Result: " + queryResult);
},

_sentinel: null
	
});

})();
