/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.page.WorkItemMailConfigPageController");

dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemMailConfigMessages");

(function(){

    var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
    
    var MSGS = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemMailConfigMessages"); //$NON-NLS-1$ //$NON-NLS-2$
    
    /* Main Work Item Mail Configuration page controller */
    dojo.declare("com.ibm.team.workitem.web.ui.internal.page.WorkItemMailConfigPageController", null, {
    
        constructor: function(workItemMailConfigView){
            this._workItemMailConfigView = workItemMailConfigView;
        },
        
        /* externalized messages for the template */
        _messages: MSGS,
        
        _loadAndShowMailConfig: function(){
            //console.debug("_loadAndShowMailConfig()");
            
            var serviceRequest = new ServiceRequest("com.ibm.team.workitem.common.internal.rest.IWorkItemRestService", "getContributorMailConfigDTO", {
                uuid: this._workItemMailConfigView.itemUUID
            });
            var self = this;
            var handlerObj = {
                success: function(result){
                    self._workItemMailConfigView._setInput(result);
                },
                failure: function(errorObj){
                    alert(errorObj.message);
                }
            }
            
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            TeamServerClient.invokeService(serviceRequest, responseHandler);
        },
        
        _performSave: function(config, callback){
            //console.debug("_performSave()");
            
            var serviceRequest = new ServiceRequest("com.ibm.team.workitem.common.internal.rest.IWorkItemRestService", "postContributorMailConfig", {
                configuration: dojo.toJson(config),
                uuid: this._workItemMailConfigView.itemUUID
            });
            var self = this;
            var handlerObj = {
                success: function(){
                    callback(self._workItemMailConfigView.pageID);
                    self._workItemMailConfigView._onModified(false);
                },
                failure: function(errorObj){
                    callback(self._workItemMailConfigView.pageID, new Error(self._messages.SAVE_ERROR + errorObj.message));
                }
            }
            
            var responseHandler = new ServiceResponseHandler(handlerObj, "success", "failure");
            TeamServerClient.invokeService(serviceRequest, responseHandler);
        }
    });
})();
