/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojo.fx"); //$NON-NLS-1$
dojo.require("dojo.fx.easing"); //$NON-NLS-1$
dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.dialogs.SaveErrorDialog"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.view.plan.actions.Action"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var Action= com.ibm.team.apt.web.ui.internal.view.plan.actions.Action;
var SaveErrorDialog= com.ibm.team.apt.web.ui.internal.view.plan.dialogs.SaveErrorDialog;
var SelectAction= com.ibm.team.apt.web.ui.internal.view.plan.actions.SelectAction;
	
var doc= document;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

var removeChildren= dojox.data.dom.removeChildren;

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar", null, { //$NON-NLS-1$

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------

	__viewModelViewer: null,
	__viewModelViewerItem: null,
	__viewModel: null,
	__actionProviderFn: null,
	
	__showAsSelected: false,
	
	constructor: function(viewModelViewerItem, actionProviderFn) {
		this.__viewModelViewerItem= viewModelViewerItem;
		this.__viewModelViewer= viewModelViewerItem.getViewer();
		this.__viewModel= this.__viewModelViewerItem.getViewModel()
		this.__viewEntry= this.__viewModelViewerItem.getViewEntry()
		
		this.__actionProviderFn= actionProviderFn;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	refresh: function() {
		dojo.toggleClass(this._node, "selected", this.__viewModelViewerItem.isSelected() || this.__showAsSelected); //$NON-NLS-1$
		dojo.toggleClass(this._node, "single", this.__viewModelViewer.isSingleSelection()); //$NON-NLS-1$
		this.__refreshPopup();
	},

	// ---- PlanItemViewerItem api -----------------------------------------------------------------------------------------

	_createContainer: function() {
		this.__createContent();
		this.refresh();
		return this._node;
	},
	
	_dispose: function() {
		this.__disposeToolbar();

		if (this.__hoverListener != null) {
			dojo.disconnect(this.__hoverListener);
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	__createContent: function() {
		var localDoc= doc;
		
		var node= localDoc.createElement("div"); //$NON-NLS-1$
		dojo.addClass(node, "menuIcon"); //$NON-NLS-1$
		
		this.__hoverListener= dojo.connect(node, "mouseover", this, this._onMouseOver); //$NON-NLS-1$
		this.__clickListener= dojo.connect(node, "onclick", this, this._onMouseClick); //$NON-NLS-1$
		this._node= node;
	},

	_onMouseOver: function(e) {
		var popup= this.__getPopup();
		popup.open();
		
		if (this.__openedEmptyTooltip)
			popup.close();
	},

	_onMouseClick: function(e) {
		this.__viewModel.setEntrySelectionState(this.__viewModelViewerItem.getViewEntry(), !this.__viewModelViewerItem.isSelected());
	},

	__getPopup: function() {
		if (!this.__popup) {
			this.__popup= new com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar._ToolbarPopup({
				openDelay: 0,
				closeDelay: 100,
				around: [this._node], //$NON-NLS-1$
				orientation: "R", //$NON-NLS-1$
				onOpen: dojo.hitch(this, this.__popupOpened),
				onClose: dojo.hitch(this, this.__popupClosed),
				__actionBar: this
			});
		}
		return this.__popup;
	},
	
	__popupOpened: function(popup) {
		this.__popupContent= popup.content;
		this.__openedEmptyTooltip= !this.__refreshPopup();

		// avoid double listeners
		this.__removePopupHoverListener();
		this.__popupHoverListener= dojo.connect(jazz.ui._getMasterPopup().domNode, "mouseover", this, this._onMouseOverPopup); //$NON-NLS-1$
		dojo.addClass(this._node, "hot"); //$NON-NLS-1$
	},

	__popupClosed: function() {
		dojo.toggleClass(this._node, "selected", this.__viewModelViewerItem.isSelected()); //$NON-NLS-1$
		this.__popupContent= null;
		this.__removePopupHoverListener();
		this.__showAsSelected= false;
		this.__viewModelViewerItem.getViewer().setSingleSelection(false);
		dojo.removeClass(this._node, "hot"); //$NON-NLS-1$
	},

	__removePopupHoverListener: function() {
		if (this.__popupHoverListener) {
			dojo.disconnect(this.__popupHoverListener);
			this.__popupHoverListener= null;
		}
	},
	
	_onMouseOverPopup: function(e) {
		var showAsSelected= false;
		var showAsSingleSelected= false;
		
		var action= null;
		var targetNode= e.target;
		while (targetNode != null) {
			if (dojo.hasClass(targetNode, "action")) { //$NON-NLS-1$
				action= this.__toolbar.getAction(targetNode.getAttribute("actionId")); //$NON-NLS-1$
				break;
			}
			targetNode = targetNode.parentNode;
		}
		
		if (action) {
			if ((action.getStyle() & Action.STYLE_SHOW_AS_SINGLESELECTED) != 0) {
				showAsSelected= showAsSingleSelected= true;
			} else {
				if ((action.getStyle() & Action.STYLE_SHOW_AS_SELECTED) != 0) {
					showAsSelected= true;
				} else {
					showAsSelected= this.__viewModelViewerItem.isSelected();
				}
			}
		}
		this.__showAsSelected= showAsSelected;
		this.__viewModelViewerItem.getViewer().setSingleSelection(showAsSingleSelected);
		this.refresh();
	},

	__onMenuOpen: function(menu, menuCloseFn) {
		this.__menuCloseFn= menuCloseFn;
	},
	
	__onMenuClose: function() {
		this.__menuCloseFn= null;
		this.__popup.close();
	},
	
	__onExecute: function() {
		this.__popup.close();
	},

	__refreshPopup: function() {
		var hasEnabledActions= false;
		if (this.__popupContent != null) {
			
			if (this.__toolbar) {
				hasEnabledActions= this.__toolbar.update();
			} else {
				this.__toolbar= new com.ibm.team.apt.web.ui.internal.view.plan.actions.ToolBarManager();
				dojo.forEach(this.__actionProviderFn(), function(action){
					this.__toolbar.add(action);
				}, this);					
				this.__toolbar.fillToolBar(this.__popupContent);
				hasEnabledActions= this.__toolbar.update();
				
				this.__menuOpenListener= dojo.connect(this.__toolbar, "onMenuOpen", this, this.__onMenuOpen); //$NON-NLS-1$
				this.__menuCloseListener= dojo.connect(this.__toolbar, "onMenuClose", this, this.__onMenuClose); //$NON-NLS-1$
				this.__menuCloseListener= dojo.connect(this.__toolbar, "onExecute", this, this.__onExecute); //$NON-NLS-1$
			}
		}
		
		return hasEnabledActions;
	},
	
	__disposeToolbar: function() {
		if (this.__toolbar != null) {
			if (this.__menuOpenListener) {
				dojo.disconnect(this.__toolbar, this.__menuOpenListener);
				this.__menuOpenListener= null;
			}
			
			if (this.__menuCloseListener) {
				dojo.disconnect(this.__toolbar, this.__menuCloseListener);
				this.__menuCloseListener= null;
			}
			this.__toolbar= null;
		}
	},

	__sentinel: null // terminates this class definition
});

var DEFAULT_MAX_WIDTH = 500;
var DEFAULT_MAX_HEIGHT = 350;

var table= null;
var tablePos= {x: 230, y: 230};

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.ViewModelViewerItemActionBar._ToolbarPopup", jazz.ui.HoverPopup, { //$NON-NLS-1$
	
	position: function(node) {
	
		var master = jazz.ui._getMasterPopup();
		if (!node) {
			if (!this._isOpen || !this._target)
				return;
			node = this._target;
		}
		
		tablePos= dojo._abs(node, true);

		var pos= {
				x: tablePos.x + 25,
				y: tablePos.y 
		}
		
		master.domNode.style.zIndex= "999"; //$NON-NLS-1$

		master.domNode.style.left= (pos.x - 8) + "px"; //$NON-NLS-1$
		master.domNode.style.top = (pos.y - 6) + "px"; //$NON-NLS-1$
		master.arrow.style.display = "none"; //$NON-NLS-1$

		master.contentWrapper.scrollTop = 1; // workaround a scrollbar display bug in FF
		master.contentWrapper.scrollTop = 0;
	},

	close: function() {
		if (this.__actionBar.__menuCloseFn) {
			this.__actionBar.__menuCloseFn();
			this.__actionBar.__menuCloseFn= null;
		}

		this.inherited("close", arguments, arguments); //$NON-NLS-1$
	},
	
	_hover: function() {
		// overriden to avoid default implementation
	},
	
	_unHoverPopup: function() {
		if (this.__actionBar.__menuCloseFn)
			return;

		this.inherited("_unHoverPopup", arguments, arguments); //$NON-NLS-1$
	},

	__sentinel: null // terminates this class definition
});


})();
