/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.DiscussionWidget");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment");
dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.DiscussionWidget",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/layout/templates/DiscussionWidget.html"),
        constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WidgetMessages");
			this.textCollapseAll = this.messages.buttonCollapseAll;
			this.textExpandAll = this.messages.buttonExpandAll;
			this.textAddComment = this.messages.buttonAddComment;
		},
		postCreate: function() {
			this.widgetCache = {};
			if(!this.attributeId) this.attributeId="comment";
			if(this.attributeId != "comment") {
				console.debug(dojo.string.substitute(this.messages.errorBadWidgetAttribute, [this.declaredClass, this.messages.labelNoAttribute]));
				this.attributeId="comment";
			}
			if(this.creationSpec && this.creationSpec.iscommentManager) this.iscommentManager = true; // TODO: defect 46775
			
			var comments = this.workingCopy.getValue({
				path: "attributes.internalComments.items"
			});
			if (!comments) { comments= { length: 0 }; }

			this._updateCommentCount(comments.length);
			if(comments.length == 0) {
				this._expandCollapseDiv.style.display = "none";
				var noComments = document.createElement("div");
				noComments.className = "NoComments";
				noComments.appendChild(document.createTextNode(this.messages.labelNoComments));
				this.discussion.appendChild(noComments);				
				dojo.removeClass(this.discussion, "discussion-background");
			} else  {				
				this._expandCollapseDiv.style.display = "";
				dojo.addClass(this.discussion, "discussion-background");
				var workItem = this.workItem;
				if(!workItem && this.workingCopy) workItem = this.workingCopy.object;
				if(!workItem && this.parentWidget && this.parentWidget.workingCopy) workItem = this.parentWidget.workingCopy.object;
				if(!workItem && this.parentController && this.parentController.workingCopy) workItem = this.parentController.workingCopy.object; 
				for (var i=0; i<comments.length; i++) {
					comments[i].commentNumber = i + 1;
					var commentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment( {	
					    parentWidget: this.parentWidget,
					    parentController: this.parentController,
						comment: comments[i],
						editorUtil: this.parentWidget.editorUtil,
						workItem: workItem
					 } );
					 this.widgetCache[comments[i].commentNumber] = commentWidget;
					 if(this._modifiers.expanded == "false") {
					 	  this._doExpandCollapse(/* isexpanded */ false)
					 }
//					comments[i].commentWidget = commentWidget;
					this.discussion.appendChild(commentWidget.domNode);
				}
			}			

			// add another comment for the "new comment" area
			commentWidget = /* createWidget */ new com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemComment( {
				workingCopy: this.workingCopy,
				attributeId: this.attributeId,
				parentWidget: this,	
				parentController: this.parentController,
				comment: null,
				editorUtil: this.parentWidget.editorUtil
				 } );
				 
			if(this.sectionLayout) this.sectionLayout.setBorder(true);
			this.isCommentShowing = false;
			if (!this.workingCopy.isGuest()) {
				if(this.sectionLayout) this._addCommentAction = this.sectionLayout.addActionText(this.messages.buttonAddComment, this, "_showNewCommentArea");
			}
			else {
				this.createCommentCmdBottom.style.visibility = "hidden";
			}
		    commentWidget._saveCmdBottom = this.parentWidget._saveCmdBottom; //CHANGE TO FUNCTION
			this.newCommentDiv.appendChild(commentWidget.domNode);
			this.newCommentArea = commentWidget;
			this.comment = commentWidget.commentText; // Bridge the form element
			
			this.workingCopy.addListener({
				path: ["attributes", this.attributeId],
				event: "onchange",
				listener: this,
				functionName: "_handleAttributeChange"
			});			
			this._handleAttributeChange();
		},
		getDefaultLabel: function() {
			return this.messages.labelDiscussion;
		},
		_handleAttributeChange: function(parms) {
			if(parms && parms.caller == this.newCommentArea) return; // change in the new comment widget
			var value = this.workingCopy.getValue({path:"attributes.comment"});
			var content = value ? value.content : "";
			this.setNewCommentTextValue(content);
		},
		_setAttribute: function(value, isInit){
			this.workingCopy.setValue({
				caller: this,
				path: ["attributes", this.attributeId, "content"],
				savedAttributePath: ["attributes", this.attributeId, "content"],
				attributeId: this.attributeId,
				isInitialization: isInit,
				value: value
			});
		},
		destroyWidget: function() {
			if(this.newCommentArea) {
				this.newCommentArea.destroyWidget();
			}
			if(this.widgetCache) {				
				for(var key in this.widgetCache) {
					var widget = this.widgetCache[key];
					if(widget) {					
						if(widget.destroyWidget) {
							widget.destroyWidget();
						} else {
							if(widget.destroy) widget.destroy();
						}
					}
				}
			}
			this.widgetCache = {};
			this.destroy();
		},		
		clearNewCommentTextValue: function() {
			this.setNewCommentTextValue("");
		},
		setNewCommentTextValue: function(value) {
			if(value && value.length > 0) {
				this._showNewCommentArea(true);				
			}
			this.newCommentArea.setTextValue(value);
		},
		getNewCommentTextValue: function () {
			var result =this.newCommentArea.getTextValue();
			if(result == "\n") return ""; // dijit.textarea bug
			return  result;
		},
		_updateCommentCount: function(count) {
			if(!this.sectionLayout)  return;
					if(count > 1) {
						this.sectionLayout.setMessage(dojo.string.substitute(this.messages.labelSomeComments, [("" + count)]));						
					} else if(count == 1) 
						{
							this.sectionLayout.setMessage(this.messages.labelOneComment);
						} else 
						{
							this.sectionLayout.setMessage("")
						} 
		},
		getValue: function() {
			return this.getNewCommentTextValue();
		},
		markFullyLoaded: function() {
			if(this.getNewCommentTextValue().length > 0) {
				this._showNewCommentArea(true); // give it focus
			}
		}, 
		_handleChange: function(e) {
			this.parentWidget._handleDirtyingEvent(e);
		},
		_handleNewCommentClick:function(e) {
			e.preventDefault();
			this._showNewCommentArea();
		},
		_showNewCommentArea: function(alwaysShow) {
			if(this.sectionLayout && !this._addCommentAction) return;
			if(alwaysShow || !this.isShowingComment){
				this.isShowingComment = true;
				this.newCommentArea.showNewComment();
				dojo.addClass(this.createCommentCmdBottom, "SectionActionAnchorDisabled");
//				dojox.data.dom.replaceChildren(this.createCommentCmdBottom, document.createTextNode(this.messages.buttonHideComment));
				if(this._addCommentAction){
					dojo.addClass(this._addCommentAction._jazzAnchor, "SectionActionAnchorDisabled");
//					dojox.data.dom.replaceChildren(this._addCommentAction._jazzAnchor, document.createTextNode(this.messages.buttonHideComment));	
				}
			}
			 // Once shown the comment area cannot be hidden
//			 else {
//				this.isShowingComment = false;
//				this.newCommentArea.hideNewComment();
//				var text;
//				if(this.getNewCommentTextValue().length > 0) {
//					text = this.messages.buttonShowComment;
//				} else {
//					text = this.messages.buttonAddComment;
//				}
//				
//				dojox.data.dom.replaceChildren(this.createCommentCmdBottom, document.createTextNode(text));
//				if(this._addCommentAction){
//					dojox.data.dom.replaceChildren(this._addCommentAction._jazzAnchor, document.createTextNode(text));	
//				}
//			}
		},
		_handleCollapseAll: function(e) {
			dojo.stopEvent(e);
			this._doExpandCollapse(false);
		},
		_handleExpandAll: function(e) {
			dojo.stopEvent(e);
			this._doExpandCollapse(true);
		},
		_doCommentCommand: function(commandName ){
			var comments= this.workingCopy.getValue({path: "attributes.internalComments.items", attributeId: "internalComments"}) || []; 
			if (!comments || comments.length == 0) return;
			for (var i=0; i<comments.length; i++) {
				if(this.widgetCache[comments[i].commentNumber]) {
					if(this.widgetCache[comments[i].commentNumber][commandName]) {
						this.widgetCache[comments[i].commentNumber][commandName]();
					}
				}
			}			
		},
		_doExpandCollapse: function(doExpand) {
				if (doExpand) {
					this._doCommentCommand("expand");
				} else {
					this._doCommentCommand("collapse");
				}
		},
		clearRetainedState: function() {
			this.clearNewCommentTextValue();
		},
		hasUnsavedChanges: function() {
			return this.getNewCommentTextValue().length > 0
		},
		handleSaveComplete: function() {
			this.clearNewCommentTextValue();
		}
	}
);

})();
