/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.search.IterationPlanBanner"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

(function() {

var checkNotNull 		= net.jazz.ajax.toolbox.lang.checkNotNull;
var checkNotEmpty 		= net.jazz.ajax.toolbox.lang.checkNotEmpty;

var PlanningUtil		= com.ibm.team.apt.web.client.util;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * Provides a UI to iterate through the iteration plans in a query result.
 */
dojo.declare("com.ibm.team.apt.web.ui.internal.view.search.IterationPlanBanner", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_searchResults: null,
	_currentPlan: null,
	
	_breadcrumbs: null,
	_pageNumberContainer: null,

	constructor: function() {
		
	},

	// ---- api ------------------------------------------------------------------------------------------------------------	


	// ---- widget implementation ------------------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/search/templates/IterationPlanBanner.html"), //$NON-NLS-1$ //$NON-NLS-2$

	postCreate: function() {
		var currentIndex = this._getCurrentIndex();
		
		if(currentIndex === -1) {
			this.domNode.style.display = "none"; //$NON-NLS-1$
			return;
		} else {
			this.domNode.style.display = "block"; //$NON-NLS-1$
		}

		var queryDescriptor= this._searchResults.getQueryDescriptor();

		var anchor = document.createElement("A"); //$NON-NLS-1$
		anchor.href = PlanningUtil.getSearchURI(queryDescriptor, false);
		anchor.appendChild(document.createTextNode(queryDescriptor.getLabel()));
		this._breadcrumbs.appendChild(anchor);
		this._breadcrumbs.appendChild(document.createTextNode(" > ")); //$NON-NLS-1$
		
		this._pageNumberContainer.style.display="block"; //$NON-NLS-1$
		// show previous page text or link
		this._pageNumberContainer.appendChild(this._getPreviousElement(currentIndex));
		
		// show count info
		var pageInfoSpan= document.createElement("span"); //$NON-NLS-1$
		pageInfoSpan.className= "pageInfoSpan"; //$NON-NLS-1$
		pageInfoSpan.innerHTML= bind(MSGS.BANNER_PAGEINFO, [currentIndex + 1, this._searchResults.getResultCount()]);
		
		this._pageNumberContainer.appendChild(pageInfoSpan);
		
		// show next page link or text
		this._pageNumberContainer.appendChild(this._getNextElement(currentIndex));
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------

	_getCurrentIndex: function() {
		return this._searchResults.getIndex(this._currentPlan);
	},

	_getPreviousElement: function(currentIndex) {
		if(currentIndex > 0) {
			return this._getLinkElement(currentIndex - 1, MSGS.BANNER_PREVIOUS, MSGS.BANNER_PREVIOUS_TITLE);
		}

		return	this._getDisabledTextElement(MSGS.BANNER_PREVIOUS);
	},
	
	_getNextElement: function(currentIndex) {
		if(currentIndex < this._searchResults.getSearchResults().length - 1) {
			return this._getLinkElement(currentIndex + 1, MSGS.BANNER_NEXT, MSGS.BANNER_NEXT_TITLE);
		}

		return	this._getDisabledTextElement(MSGS.BANNER_NEXT);
	},

	_getLinkElement: function(targetIndex, label, title) {
		var topDiv= document.createElement("SPAN"); //$NON-NLS-1$

		var searchResults= this._searchResults.getSearchResults();
		var plan= searchResults[targetIndex];
	
		var linkElement = document.createElement("a"); //$NON-NLS-1$
		linkElement.href = PlanningUtil.getPlanViewURI(plan.itemId);
	 	linkElement.title = bind(title, [plan.label, plan.getIteration().label]);

		linkElement.appendChild(document.createTextNode(label));
		
		topDiv.appendChild(linkElement);
		
		return topDiv;
	},

	_getDisabledTextElement: function(str) {
		var span= document.createElement("span"); //$NON-NLS-1$
		span.className= "disabledText"; //$NON-NLS-1$
		span.appendChild(document.createTextNode(str));
		return span;
	},

	__sentinel: null // terminates this class definition

});

/**
 * This helper method creates the arguments object for the widget's constructor.
 */
com.ibm.team.apt.web.ui.internal.view.search.IterationPlanBanner.constructorArgs= function(currentPlan, searchResults) {
	return {
		_currentPlan: currentPlan,
		_searchResults: searchResults
	};
};


})();