/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.viewers.ActionColumnLabelProvider"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.viewers.DefaultLabelProvider"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

(function() {
	
var Assert			= com.ibm.team.apt.web.ui.internal.common.Assert;
	
dojo.declare("com.ibm.team.apt.web.ui.internal.viewers.ActionColumnLabelProvider", //$NON-NLS-1$
			 [com.ibm.team.apt.web.ui.internal.viewers.DefaultLabelProvider], {
	
	__handlers: null,
	
	constructor: function() {
		this.__handlers= new Array();
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	getColumn: function(element, index) {
		if (index == this._getActionColumnIndex()) {
			return this.__createActionBar(element);
		} else {
			return this.inherited(arguments);
		}
	},
		
	destroy: function() {
		for ( var i = 0; i < this.__handlers.length; i++) {
			dojo.disconnect(this.__handlers[i]);
		}
		
		this.inherited(arguments);
	},
	
	// ---- SPI --------------------------------------------------------------------------------------------------
	
	_getActionColumnIndex: function() {
		Assert.isLegal(false, "Subclass must implement ActionColumnLableProvider#_getActionColumnIndex"); //$NON-NLS-1$
		return 0;
	},
	
	_createActions: function(element) {
		Assert.isLegal(false, "Subclass must implement ActionColumnLableProvider#_createActions"); //$NON-NLS-1$
		return new Array();
	},
	
	_createAction: function(label, image, callback) {
		var result= document.createElement("a"); //$NON-NLS-1$
		result.setAttribute("title", label); //$NON-NLS-1$
		result.href = "#"; //$NON-NLS-1$
		
		//Make the child of the link a real image so that we can pick up it's alt text when images are off
		var img= document.createElement("img"); //$NON-NLS-1$
		img.src= image; //$NON-NLS-1$ //$NON-NLS-2$
		img.alt= label;
		result.appendChild(img);
		
		var handler= dojo.connect(result, "onclick", function(event) { //$NON-NLS-1$
			dojo.stopEvent(event);
			callback();
		});
		this.__handlers.push(handler);
		
		return result;
	},
	
	// ---- implementation -------------------------------------------------------------------------------------------------
	
	__createActionBar: function(element) {
		
		var span = document.createElement("span"); //$NON-NLS-1$
		span.tabIndex = "0"; //$NON-NLS-1$
		
		var actionTable= document.createElement("div"); //$NON-NLS-1$
		dojo.addClass(actionTable, "aptActionBar"); //$NON-NLS-1$
		
		var actions= this._createActions(element);
		for ( var i = 0; i < actions.length; i++) {
			actionTable.appendChild(actions[i]);
		}
		
		/*
		 * On focus set the visibility of the the action bar.
		 */
		dojo.connect(span, "onfocus", dojo.hitch(this, function(event) { //$NON-NLS-1$
			dojo.stopEvent(event);		
			actionTable.style.visibility = "visible"; //$NON-NLS-1$
			if(actions.length > 0){
				actions[0].focus();
			}	
		}));	
				
		
		//clear the table when blurring the last child. Commented out as it affects visibility with the mouse
		//if(actions.length > 0){
		//	dojo.connect(actions[actions.length-1],"onblur",dojo.hitch(this, function(event) { //$NON-NLS-1$
		//		dojo.stopEvent(event);	
		//		actionTable.style.visibility = "hidden";
				
		//	}));
		//}	
		
		span.appendChild(actionTable);
		return span;
	},
	
	__sentinel: null // terminates this class definition
});

})();
