/*
 * Licensed Materials - Property of IBM
 * Rational ClearQuest 
 * (C) Copyright IBM Corp. 2009  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 */
/*
 * Enhancement to dijit.Dialog including:
 * - Show icons in the title bar
 * - Show loading icons in the title bar
 */
dojo.provide("com.ibm.rational.cm.web.ui.view.common.Dialog");

dojo.require("dijit.Dialog");

dojo.declare(
	"com.ibm.rational.cm.web.ui.view.common.Dialog",
	dijit.Dialog,
{
	iconClass: "",
	parentDialog: null,
	
	postCreate: function(){
		dojo.addClass(this.domNode, "com-ibm-rcm-Dialog");
		
		if(this.iconClass.length > 0){
			this.setIcon(this.iconClass);
		}
		this.inherited(arguments);
		this.setTitle(this.title);
		
		this.setParentDialog(this.parentDialog);
	},
	
	_setup: function(){
		this.inherited(arguments);
		dojo.addClass(this._underlay.domNode, "com-ibm-rcm-DialogUnderlay");
	},
	
	setTitle: function(/* String */ title){
		this.title = title;
		this.titleNode.innerHTML = " "+title;
	},
	
	setIcon: function(/* String */ iconClass){
		var iconNode = this._iconNode;
		if(!iconNode){
			iconNode = this._iconNode = dojo.doc.createElement("SPAN");
			dojo.addClass(iconNode, "dijitInline");
			this.titleBar.insertBefore(iconNode, this.titleNode);
		}
		if(iconClass && iconClass.length > 0){
			dojo.addClass(iconNode, iconClass);
		}else{
			iconNode.className = "dijitInline";
		}
	},
	
	setLoading: function(/* boolean */ loading){
// remove spinner
/*
		var loadingNode = this._loadingNode;
		var iconNode = this._iconNode;
		if(loading){
			if(!loadingNode){
				loadingNode = this._loadingNode = dojo.doc.createElement("SPAN");
				dojo.addClass(loadingNode, "dijitInline");
				dojo.addClass(loadingNode, "loadingIcon");
				var beforeNode = (this._iconNode) ? this._iconNode : this.titleNode;
				this.titleBar.insertBefore(loadingNode, beforeNode);
			}else{
				loadingNode.style.display = "";
			}
			if(iconNode){
				iconNode.style.display = "none";
			}
		}else if(loadingNode){
			loadingNode.style.display = "none";
			if(iconNode){
				iconNode.style.display = "";
			}
		}
		*/
	},
	
	/*
	 * If parent dialog is set:
	 * - hide parent dialog when this dialog is shown
	 * - show parent dialog when this dialog is hidden
	 * so that this dialog won't get hidden under parent dialog
	 * 71281: Query chooser dialog is hidden under "Search and select records" dialog
	 */
	setParentDialog: function(/* dijit.Dialog */ dlg){
		// first disconnect
		if(this._parentConnects){
			dojo.forEach(this._parentConnects, dojo.hitch(this, "disconnect"));
		}
		
		if(dlg && dlg.show && dojo.isFunction(dlg.show) && dlg.hide && dojo.isFunction(dlg.hide)){
			this._parentConnects = [
				this.connect(this, "show", dojo.hitch(dlg, "hide")),
				this.connect(this, "hide", dojo.hitch(dlg, "show"))
			];
		}
	}
});