/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.PlanElementDelta"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanElementAttributeDelta"); //$NON-NLS-1$

(function() {
	
var KIND_MASK 						= 0xFF;
var NO_CHANGE						= 0x0;
var ADDED							= 0x1;
var REMOVED							= 0x2;
var CHANGED							= 0x4;

var CONTENT							= 0x100;
var STRUCTURE						= 0x200;

var PlanElementAttributeDelta		= com.ibm.team.apt.client.PlanElementAttributeDelta;

dojo.declare("com.ibm.team.apt.client.PlanElementDelta", null, { //$NON-NLS-1$

	__status: null,
	__parentDelta: null,
	__planElement: null,
	__attributeDeltas: null,
	__children: null,
	__moveParent: null,
	__otherMoveDelta: null,

	constructor: function(parentDelta, planElement) {
		this.__parentDelta= parentDelta;
		this.__planElement= planElement;
		this.__status= NO_CHANGE;
	},
	
	// ---- api ------------------------------------------------------------------------------------------------------------

	/**
	 * @type PlanElementDelta
	 */
	getParentDelta: function() {
		return this.__parentDelta;
	},

	/**
	 * @type boolean
	 */
	isAdded: function() {
		return (this.__status & ADDED) != 0;
	},

	/**
	 * @type boolean
	 */
	isRemoved: function() {
		return (this.__status & REMOVED) != 0;
	},

	/**
	 * @type boolean
	 */
	isDeleted: function() {
		return this.isRemoved() && this.getMovedTo() === null;
	},

	/**
	 * @type boolean
	 */
	isChanged: function() {
		return (this.__status & CHANGED) != 0;
	},

	/**
	 * @type boolean
	 */
	isContentChange: function() {
		return (this.__status & CONTENT) != 0;
	},

	/**
	 * @type boolean
	 */
	isStructuralChange: function() {
		return (this.__status & STRUCTURE) != 0;
	},

	/**
	 * @type boolean
	 */
	isMoveChange: function() {
		return this.__moveParent != null;
	},

	/**
	 * @type PlanElement
	 */
	getPlanElement: function() {
		return this.__planElement;
	},

	/**
	 * @type PlanElement
	 */
	getMovedFrom: function() {
		if (!this.isAdded())
			return null;
		return this.__moveParent;
	},

	/**
	 * @type PlanElement
	 */
	getMovedTo: function() {
		if (!this.isRemoved())
			return null;
		return this.__moveParent;
	},

	/**
	 * @type PlanElementDelta
	 */
	getOtherMoveDelta: function() {
		return this.__otherMoveDelta; 
	},

	/**
	 * @param {Attribute} attribute
	 * @type boolean
	 */
	affectsAttribute: function(attribute) {
		return this.__attributeDeltas != null && this.__attributeDeltas[attribute.getId()];
	},

	/**
	 * @param {Attribute} attribute
	 * @type IPlanAttributeDelta
	 */
	getAttributeDeltas: function() {
		var result= [];

		if (this.__attributeDeltas != null) {
			for (var key in this.__attributeDeltas) {
				result.push(this.__attributeDeltas[key]);
            }
        }
		return result;
	},

	/**
	 * @type Array
	 */
	getChildren: function() {
		var result= [];
		if (this.__children != null)
			result.push.apply(result, this.__children);
		return result;
	},

	/**
	 * @param {PlanElement} element
	 * @type PlanElementDelta
	 */
	getChild: function(element) {
		if (this.__children == null)
			return null;
		
		for (var i= 0; i < this.__children.length; i++) {
			if (this.__children[i].getPlanElement() === element)
				return this.__children[i];
		}
		return null;
	},

	/**
	 * @param {Function} visitor
	 */
    accept: function(visitor, thisObj) {
        if (visitor.call(thisObj, this) && this.__children != null) {
		for (var i= 0; i < this.__children.length; i++) {
                this.__children[i].accept(visitor, thisObj);
            }
        }
    },
    
	// ---- implementation -------------------------------------------------------------------------------------------------
    
	
	isNoChange: function() {
		return (this.__status & KIND_MASK) == 0;
	},

	setAdded: function() {
		this.__status= this.__getFlags() | ADDED;
	},

	setRemoved: function() {
		this.__status= this.__getFlags() | REMOVED;
	},

	setChanged: function() {
		this.__status= this.__getFlags() | CHANGED;
	},

	setContentChange: function() {
		this.__status= this.__status | CONTENT;
	},

	setStructuralChange: function() {
		this.__status= this.__status | STRUCTURE;
	},

	setOtherMoveDelta: function(otherDelta) {
		this.__otherMoveDelta= otherDelta;
	},

	setMoveParent: function(element) {
		this.__moveParent= element;
	},

	addChild: function(delta) {
		if (this.__children == null)
			this.__children= [];
		this.__children.push(delta);
	},

	removeChild: function(delta) {
		if (this.__children != null) {
			var index= dojo.indexOf(this.__children, delta);
			if (index != -1) {
				this.__children.splice(index, 1);
			}
		}
	},

	hasChildren: function() {
		return this.__children != null && this.__children.length > 0;
	},

	addAttribute: function(delta) {
		if (this.__attributeDeltas == null) {
			this.__attributeDeltas= {};
		} else {
			var oldDelta= this.__attributeDeltas[delta.getAttribute().getId()];
			if (oldDelta) {
				delta= new PlanElementAttributeDelta(delta.getAttribute(), oldDelta.getOldValue(), delta.getNewValue(), delta.getGeneration());
			}
		}

		this.__attributeDeltas[delta.getAttribute().getId()]= delta;
	},

	getAttributeDelta: function(attribute) {
		return this.__attributeDeltas != null ? this.__attributeDeltas[attribute.getId()] : null;
	},
	
	__getFlags: function() {
		return this.__status & ~KIND_MASK;
	},

	__sentinel: null // terminates this class definition
});


})();
