/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.client.PlanElementDeltaAccessor"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanItem"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.client.PlanningAttributeIdentifier"); //$NON-NLS-1$

(function() {

var PlanItem= com.ibm.team.apt.client.PlanItem;
var PlanningAttributeIdentifier= com.ibm.team.apt.client.PlanningAttributeIdentifier;

dojo.declare("com.ibm.team.apt.client.PlanElementDeltaAccessor", null, { //$NON-NLS-1$

	__planElement: null,
	__newValues: null,
	__complexityAttributeId: null,
	
	constructor: function(planElement, newValues, complexityAttributeId) {
		this.__planElement= planElement;
		this.__newValues= newValues;
		this.__complexityAttributeId= complexityAttributeId;
	},
	
	__getValue: function(delta) {
		return this.__newValues ? delta.getNewValue() : delta.getOldValue();
	},
	
	isPlanItem: function(delta) {
		return true;
	},
	
	getChildren: function(delta) {
		return [];
	},

	getComplexity: function(delta) {
		var value= null;
		if(!this.__complexityAttributeId)
			return this.__workItemEstimatorGetComplexity(null);
		
		var attribute= this.__planElement.getPlanModel().findAttribute(new PlanningAttributeIdentifier(this.__complexityAttributeId));
		if(!attribute)
			return this.__workItemEstimatorGetComplexity(null);
			
		var attrDelta= delta.getAttributeDelta(attribute);
		return attrDelta ? this.__workItemEstimatorGetComplexity(this.__getValue(attrDelta)) : this.__workItemEstimatorGetComplexity(attribute.getValue(this.__planElement));
	},
	
	__workItemEstimatorGetComplexity: function(value){
		if(value === undefined){
			return 0;
		}
		
		if(value === null){
			return 0;
		}
			
		var number= 0;			
		if(value instanceof Object){
			if(value instanceof Number){
				return value;
			}
			if(value instanceof com.ibm.team.workitem.common.Literal){
				number= Number(value.itemId);
				if(!isNaN(number)){
					return number;
				}
				number= Number(value.label);
				if(!isNaN(number)){
					return number;
				}
			}
		}
		
		number= Number(value);
		if(isNaN(number)){
			return 0;
		} else {
			return number;
		}
	},
	
	getDuration: function(delta) {
		var attrDelta= delta.getAttributeDelta(PlanItem.DURATION);
		return attrDelta ? this.__getValue(attrDelta) : this.__planElement.getDuration();
	},

	isAuxiliaryPlanItem: function(delta) {
		var attrDelta= delta.getAttributeDelta(PlanItem.AUXILIARY_ITEM);
		return attrDelta ? this.__getValue(attrDelta): this.__planElement.isAuxiliaryPlanItem();
	},
	
	isPrimaryElement: function(entry) {
		return true;
	},	
	
	isResolved: function(delta) {
		var attrDelta= delta.getAttributeDelta(PlanItem.STATE);
		if (attrDelta && this.__planElement.getWorkflowInfo() != null) {
			var state= this.__getValue(attrDelta);
			if(state){
				return this.__planElement.getWorkflowInfo().getStateGroup(state.getId()) == com.ibm.team.apt.web.client.internal.mixins.WorkflowInfo.DONE;
			}
		}
		return this.__planElement.isResolved();
	},

	isTopLevelPlanItem: function(delta) {
		var attrDelta= delta.getAttributeDelta(PlanItem.ITEM_TYPE);
		if(attrDelta) {
			var workItemType= this.__getValue(attrDelta);
			return dojo.indexOf(this.__planElement.getPlanModel().getTopLevelWorkItemTypes(), workItemType) != -1;
		}
		return this.__planElement.isTopLevelPlanItem();
	},
	
	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.client.PlanElementDeltaAccessor.affects= function(delta) {
	var planModel= delta.getPlanElement().getPlanModel();
	var attributes= [PlanItem.DURATION.getId(), PlanItem.AUXILIARY_ITEM.getId(), PlanItem.STATE.getId(), PlanItem.ITEM_TYPE.getId(), PlanItem.WORKFLOW_ACTION.getId() ];
	var complexityAttribute= planModel.getPlanProgress().complexityAttribute;
	if(complexityAttribute && complexityAttribute.attributeId){
		attributes.push(complexityAttribute.attributeId);
	}
	var result= false;
	dojo.forEach(delta.getAttributeDeltas(), function(attrDelta) {
		var attrId= attrDelta.getAttribute().getId();
		result|= dojo.indexOf(attributes, attrId) != -1;
	});
	return result;
}

})();
