/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.model.EntryUtils"); //$NON-NLS-1$

(function() {

var EntryUtils= com.ibm.team.apt.ui.model.EntryUtils;
	
	
EntryUtils.isType= function(entry, type) {
	return EntryUtils.element(entry) instanceof type;
};

// ---- api: elements ------------------------------------------------------------------------------------------------------

EntryUtils.elements= function(entries) {
	var result= [];
	for (var i= 0; i < entries; i++) {
		result.push(entries[i].getElement());
	}
	return result;
};

EntryUtils.element= function(entry) {
	return entry != null ? entry.getElement() : null;
};

// ---- API: filter elements -------------------------------------------------------------------------------------------

EntryUtils.firstElement= function(entries) {
	return EntryUtils.element(EntryUtils.firstEntry(entries));
};

EntryUtils.lastElement= function(entries) {
	return EntryUtils.element(EntryUtils.lastEntry(entries));
};

EntryUtils.elementsOfType= function(entries, typeClass) {
	return EntryUtils.elements(EntryUtils.entriesOfType(entries, typeClass));
};

EntryUtils.firstElementOfType= function(entries, typeClass) {
	return EntryUtils.element(EntryUtils.firstEntryOfType(entries, typeClass));
};

EntryUtils.lastElementOfType= function(entries, typeClass) {
	return EntryUtils.element(EntryUtils.lastEntryOfType(entries, typeClass));
};

// ---- api: filter entries ------------------------------------------------------------------------------------------------

EntryUtils.firstEntry= function(entries) {
	return entries.length > 0 ? entries[0] : null;
};

EntryUtils.lastEntry= function(entries) {
	return entries.length > 0 ? entries[entries.length - 1] : null;
};

EntryUtils.entriesWithTag= function(entries, tag) {
	var result= [];
	for (var i= 0; i < entries; i++) {
		if (entries[i].hasTag(tag)) {
			result.push(entries[i]);
		}
	}
	return result;
};

EntryUtils.firstEntryWithTag= function(entries, tag) {
	for (var i= 0; i < entries; i++) {
		if (entries[i].hasTag(tag)) {
			return entries[i];
		}
	}
	return null;
};

EntryUtils.lastEntryWithTag= function(entries, tag) {
	for (var i= entries.length - 1; i >= 0; i--) {
		if (entries[i].hasTag(tag)) {
			return entries[i];
		}
	}
	return null;
};

EntryUtils.entriesOfType= function(entries, typeClass) {
	var result= [];
	for (var i= 0; i < entries; i++) {
		if (EntryUtils.isType(entries[i], typeClass)) {
			result.push(entries[i]);
		}
	}
	return result;
};

EntryUtils.firstEntryOfType= function(entries, typeClass) {
	for (var i= 0; i < entries; i++) {
		if (EntryUtils.isType(entries[i], typeClass)) {
			return entries[i];
		}
	}
	return null;
};

EntryUtils.lastEntryOfType= function(entries, typeClass) {
	for (var i= entries.length - 1; i >= 0; i--) {
		if (EntryUtils.isType(entries[i], typeClass)) {
			return entries[i];
		}
	}
	return null;
};

})();
