/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.model.ViewEntry"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model._EntryState"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;
var EntryState						= com.ibm.team.apt.ui.model._EntryState;

var SORTREQUEST_NONE= function(forceResort) { return forceResort ? SORTREQUEST_FULL : SORTREQUEST_NORMAL; };
var SORTREQUEST_NORMAL= function(forceResort) { return forceResort ? SORTREQUEST_FULL : SORTREQUEST_NORMAL; };
var SORTREQUEST_FULL= function(forceResort) { return SORTREQUEST_FULL; };

dojo.declare("com.ibm.team.apt.ui.model.ViewEntry", null, { //$NON-NLS-1$

	__disposed: false,
	__model: null,
	__element: null,
	__parent: null,
	__ownIndex: -1,
	__children: null,
	__removedChildren: null,
	
	__sortRequest: SORTREQUEST_NONE,

	__tags: null,

	__addedAsNodeCount: 0,
	__addedAsLeafCount: 0,

	__state: null,
	
	__unfilteredState: null,
	
	__treeNodeId: null,
	__nextChildId: 0,

	constructor: function(model, modelElement, parent) {
		Assert.isTrue(dojo.isFunction(modelElement.getElementKey), "getElementKey method is required"); //$NON-NLS-1$
	
		this.__tags= [];
		this.__state= EntryState.INITIAL;

		this.__model= model;
		this.__element= modelElement;
		this.__parent= parent;
	},

	// ---- API: read access -----------------------------------------------------------------------------------------------

	getModel: function() {
		return this.__model;
	},

	isRootEntry: function() {
		return this.__parent == null;
	},

	isVisible: function() {
		return this.__state.isVisible();
	},

	isDisposed: function() {
		return this.__disposed;
	},

	getElement: function() {
		return this.__element;
	},
	
	isAddedExplicitly: function() {
		return this.__addedAsLeafCount > 0;
	},

	hasTag: function(tag) {
		return this.__getTagEntry(tag) != null;
	},

	getTags: function(tagClass) {
		var result= [];
		for (var i= 0; i < this.__tags.length; i++) {
			if (this.__tags[i].tag instanceof tagClass) {
				result.push(this.__tags[i].tag);
			}
		}
		return result;
	},

	getOwnIndex: function() {
		if (this.__parent != null && this.__ownIndex != -1) {
			Assert.isTrue(this.__parent.__children[this.__ownIndex] === this);
		}
		return this.__ownIndex;
	},
	
	getTreePath: function() {
		var result;
		if (this.__parent == null) {
			result= [];
		} else {
			result= this.__parent.getTreePath();
		}
		
		result.push(this.getElement().getElementKey());
		return result;
	},
	

	// ---- ViewModelViewer's API (Web Only)--------------------------------------------------------------------------------

	_isCreateChildren: function() {
		return this.getElement().createsChildren && this.getElement().createsChildren(); 
	},
	
	_getTreeNodeId: function() {
		if (this.__treeNodeId == null) {
			this.__treeNodeId= this.__parent != null ? this.__parent.__generateChildTreeId() : ""; //$NON-NLS-1$
		}
		return this.__treeNodeId;
	},

	// ---- OutlineModel's API ---------------------------------------------------------------------------------------------

	_added: function(asLeaf) {
		if (asLeaf) {
			this.__addedAsLeafCount++;
		} else {
			this.__addedAsNodeCount++;
		}
	},
	
	_copy: function(newParent) {
		return { 
			entry: new com.ibm.team.apt.ui.model.ViewEntry._copy(this, newParent), 
			count: this.__addedAsLeafCount
		};
	},

	/**
	 * Notifies the entry that it is being released as either a leaf or node from the transformed model. The method
	 * returns whether this entry is still needed or not
	 * @param asLeaf
	 * @return returns true if the element is not being used anymore, and thus was transferred in the removed state
	 */
	_removed: function(asLeaf) {
		if (asLeaf) {
			this.__addedAsLeafCount--;
		} else {
			this.__addedAsNodeCount--;
		}

		Assert.isTrue(this.__addedAsLeafCount >= 0 && this.__addedAsNodeCount >= 0);

		if (this.__addedAsLeafCount == 0 && this.__addedAsNodeCount == 0) {
			Assert.isTrue(this.__children == null || this.__children.length == 0);
			this.__state= this.__state.remove();
			Assert.isLegal(this.__state && this.__state != null);
			this.__parent.__remove(this);
			return true;
		}

		return false;
	},

	_dispose: function() {
		var i;
		if (this.__children != null) {
			for (i= 0; i < this.__children.length; i++) {
				this.__children[i]._dispose();
			}
			this.__children= null;
		}

		if (this.__removedChildren != null) {
			for (i= 0; i < this.__removedChildren.length; i++) {
				this.__removedChildren[i]._dispose();
			}
			this.__removedChildren= null;
		}

		if (this.__tags != null)
			this.__tags= null;

		this.__ownIndex= -1;
		this.__disposed= true;
	},
	
	_setUnfilteredState: function(state) {
		this.__unfilteredState= state;
	},

	_getUnfilteredState: function() {
		return this.__unfilteredState;
	},

	_doStartUpdate: function() {
		return true;
	},

	_hasUpdates: function() {
		return this.__state.hasUpdates();
	},

	_collectUpdates: function(viewer, viewerUpdates) {
		this.__state.collectUpdates(viewer, this, viewerUpdates);
		return !this.__state.includesChildUpdates(this);
	},

	_processUpdate: function(updateChildren) {
		var i;
		var doUpdateChildren= arguments.length > 0 ? updateChildren : this.__state.includesChildUpdates(this);
		
		if (doUpdateChildren && this.__children != null) {
			for (i= 0; i < this.__children.length; i++) {
				this.__children[i]._processUpdate(doUpdateChildren);
			}
		}

		if (this.__removedChildren != null) {
			for (i= 0; i < this.__removedChildren.length; i++) {
				this.__removedChildren[i]._dispose();
			}
			this.__removedChildren= null;
		}

		this.__state= this.__state.getUpdatedState();
		Assert.isLegal(this.__state && this.__state != null);
	},

	_getParent: function() {
		return this.__parent;
	},

	_getChildren: function() {
		var result= [];
		if (this.__children != null) {
			result.push.apply(result, this.__children);
		}
		return result;
	},

	_add: function(childEntry, updateAccessor) {
		Assert.isLegal(childEntry.__parent === this);

		if (this.__children == null) {
			this.__children= [];
		}

		this.__children.push(childEntry);
		childEntry.__ownIndex= this.__children.length - 1;

		this._requestSortChildren(updateAccessor, false);
	},

	_show: function(updateAccessor) {
		this.__state= this.__state.show();
		Assert.isLegal(this.__state && this.__state != null);
	},

	_hide: function(updateAccessor) {
		this.__state= this.__state.hide();
		Assert.isLegal(this.__state && this.__state != null);
	},

	_update: function(properties, updateAccessor) {
		this.__state= this.__state.update(properties);
		Assert.isLegal(this.__state && this.__state != null);
		var sorter= this.__model._getSorter();
		if (this.__parent != null && sorter != null) {
			var needsResort= properties == null;
			for (var i= 0; !needsResort && i < properties.length; i++) {
				needsResort= sorter.isSorterProperty(properties[i]);
			}
			if (needsResort)
				this.__parent._requestSortChildren(updateAccessor, false);
		}
	},

	_setTag: function(tag) {
		var tagEntry= this.__getTagEntry(tag);
		if (tagEntry) {
			tagEntry.refCount++;
		} else {
			this.__tags.push({
				tag: tag,
				refCount: 1
			});
			this.__state= this.__state.update(null);
			Assert.isLegal(this.__state && this.__state != null);
		}
	},

	_clearTag: function(tag) {
		var tagEntry= this.__getTagEntry(tag);
		if (tagEntry != null) {
			if (--tagEntry.refCount === 0) {
				this.__tags.splice(dojo.indexOf(this.__tags, tagEntry), 1);
				this.__state= this.__state.update(null);
				Assert.isLegal(this.__state && this.__state != null);
			}
		}
	},

	_move: function(childEntry, newIndex, updateAccessor) {
		Assert.isLegal(childEntry._getParent() == this);
		Assert.isTrue(this.__children != null);

		var oldIndex= childEntry.getOwnIndex();
		Assert.isLegal(oldIndex >= 0 && oldIndex < this.__children.length);
		Assert.isLegal(this.__children[oldIndex] === childEntry);

		if (oldIndex != newIndex) {
			this.__children.splice(oldIndex, 1);
			var insertIndex= newIndex < oldIndex ? newIndex : newIndex - 1;
			this.__children.splice(insertIndex, 0, childEntry);

			for (var i= Math.min(insertIndex, oldIndex); i < this.__children.length; i++) {
				this.__children[i].__ownIndex= i;
			}
			this.__state= this.__state.resort();
			Assert.isLegal(this.__state && this.__state != null);

			this._requestSortChildren(updateAccessor, false);
		}
	},

	_refresh: function(updateAccessor) {
		this.__state= this.__state.refresh();
		Assert.isLegal(this.__state && this.__state != null);
	},

	_requestSortChildren: function(updateAccessor, forceResort) {
		this.__sortRequest= this.__sortRequest(forceResort);
	},

	_doPendingSort: function(updateAccessor) {
		if (this.__sortRequest !== SORTREQUEST_NONE) {
			var sorter= this.__model._getSorter();
			if (sorter != null && this.__children != null) {
				sorter.sort(this.__children, updateAccessor, this.__sortRequest === SORTREQUEST_FULL);

				var hasChanges= false;
				var indexDelta= 0;
				for (var i= 0; i < this.__children.length; i++) {
					var childEntry= this.__children[i];

					if (childEntry.__state.isAdd()) {
						// TODO new is not added :(
						// new entries were added at the end -> the sorting might have moved 
						// them to the front
						indexDelta++;
					} else if (childEntry.__ownIndex + indexDelta !== i) {
						hasChanges= true;
					}
	
					childEntry.__ownIndex= i;
				}
				if (hasChanges) {
					this.__state= this.__state.resort();
					Assert.isLegal(this.__state && this.__state != null);
				}
			}
			
			this.__sortRequest= SORTREQUEST_NONE;
		}
	},
	
	// ---- Implementation ------------------------------------------------------------------------------------------------- 

	__getTagEntry: function(tag) {
		for (var i= 0; i < this.__tags.length; i++) {
			if (this.__tags[i].tag.equals(tag)) {
				return this.__tags[i];
			}
		}
		return null;
	},
	
	__remove: function(childEntry) {
		Assert.isLegal(childEntry.__parent === this);
		Assert.isTrue(this.__children != null);

		if (this.__removedChildren == null) {
			this.__removedChildren= [];
		}

		this.__removedChildren.push(childEntry);

		var oldIndex= childEntry.__ownIndex;
		childEntry.__ownIndex= -1;
		Assert.isTrue(this.__children[oldIndex] === childEntry);

		this.__children.splice(oldIndex, 1);
		for (var i= oldIndex; i < this.__children.length; i++) {
			this.__children[i].__ownIndex= i;
		}
	},

	_accept: function(visitor, visitDeleted, thisObj) {
		var i;
		if (visitor.call(thisObj, this)) {
			if (visitDeleted && this.__removedChildren != null) {
				for (i= 0; i < this.__removedChildren.length; i++) {
					this.__removedChildren[i]._accept(visitor, visitDeleted, thisObj);
				}
			}

		 	if (this.__children != null) {
				for (i= 0; i < this.__children.length; i++) {
					this.__children[i]._accept(visitor, visitDeleted, thisObj);
				}
			}
		}
	},

	__generateChildTreeId: function() {
		var result= this._getTreeNodeId();
		if (result != "") { //$NON-NLS-1$
			result+= "-"; //$NON-NLS-1$
		}
		
		return result + (this.__nextChildId++);
	},
	
	__sentinel: null // terminates this class definition
});

com.ibm.team.apt.ui.model.ViewEntry._copy= function(other, parent) {
	Assert.isTrue(!other.__disposed);
	
	var result= new com.ibm.team.apt.ui.model.ViewEntry(other.__model, other.__element, parent);
	result.__sortRequest= other.__sortRequest;
	result.__tags= dojo.clone(other.__tags);
	result.__unfilteredState= other.__unfilteredState;
	return result;
};


})();
