/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.ui.model.ViewModel"); //$NON-NLS-1$

dojo.require("com.ibm.team.foundation.Assert"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.client.PlanItem");  //$NON-NLS-1$

dojo.require("com.ibm.team.apt.ui.model.EntryNavigator"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.UnfilteredChildrenTag"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model.ViewEntry"); //$NON-NLS-1$
dojo.require("com.ibm.team.apt.ui.model._ViewerUpdater"); //$NON-NLS-1$

(function() {

var Assert							= com.ibm.team.foundation.Assert;

var PlanItem						= com.ibm.team.apt.client.PlanItem;

var EntryNavigator					= com.ibm.team.apt.ui.model.EntryNavigator;
var UnfilteredChildrenTag			= com.ibm.team.apt.ui.model.UnfilteredChildrenTag.INSTANCE;
var ViewEntry						= com.ibm.team.apt.ui.model.ViewEntry;

dojo.declare("com.ibm.team.apt.ui.model.ViewModel", null, { //$NON-NLS-1$
	
	__viewer: null,

	__input: null,
	__modelTransformer: null,
	__filters: null,
	__sorter: null,

	__rootEntry: null,
	__reverseItemMap: null,

	__generationCounter: 0,
	__nestedUpdates: 0,
	__endUpdateRunnables: null,

	__postUpdateBeforeFilterRunnables: null,
	__postUpdateAfterFilterRunnables: null,

	/**
	 * Selection manager, manages selections of type com.ibm.team.apt.ui.model.ViewEntry[] and selection change listeners
	 */
	__selectionManager: null,

	constructor: function() {
		this.__filters= [];
		this.__reverseItemMap= {};
		this.__postUpdateBeforeFilterRunnables= [];
		this.__postUpdateAfterFilterRunnables= [];
		this.__selectionManager= {
				__selection: null,
				__listeners: null,
				
				viewerChanges: function(oldViewer, newViewer) {},

				// ---- ISelectionProvider implementation --------------------------------------------------------------------------

				addSelectionChangedListener: function(listener) {
					if (!this.__listeners)
						this.__listeners= [];
					
					this.__listeners.push(listener);
				},
				
				removeSelectionChangedListener: function(listener) {
					if (!this.__listeners)
						return null;
					
					for ( var i = 0; i < this.__listeners.length; i++) {
						if (this.__listeners[i] == listener) {
							this.__listeners.splice(i, 1);
							return null;
						}
					}
				},
				
				getSelection: function() {
				},
				
				setSelection: function(selection) {	
					this.selectionChanged({selection: this.__selection });
				},

				// ---- ISelectionChangedListener implementation -------------------------------------------------------------------

				selectionChanged: function(viewerEvent) {
					if (!this.__listeners)
						return null;
					
					for ( var i = 0; i < this.__listeners.length; i++) {
						this.__listeners[i](viewerEvent);
					}
				}
			};
	},

	// ---- API ------------------------------------------------------------------------------------------------------------

	setViewer: function(newViewer) {
		var oldViewer= this.__viewer;
		this.__viewer= newViewer;
		this.__selectionManager.viewerChanges(oldViewer, newViewer);
		this.__viewer.setInput(this);
		this.refresh();
	},

	setInput: function(input) {
		this.__input= input;
		if (this.__modelTransformer != null) {
			this.__modelTransformer.inputChanged(this, this.__input);
		}
		if (this.__sorter != null) {
			this.__sorter.inputChanged(this, this.__input);
		}
		this.refresh();
	},

	getInput: function() {
		return this.__input;
	},

	getModelTransformer: function() {
		return this.__modelTransformer;
	},
	
	setModelTransformer: function(transformer) {
		if (this.__modelTransformer != null) {
			this.__modelTransformer.inputChanged(null, null);
		}

		this.__modelTransformer= transformer;

		if (this.__modelTransformer != null) {
			this.__modelTransformer.inputChanged(this, this.getInput());
		}

		this.refresh();
	},

	viewerPreservingRefresh: function() {
		// TODO we should really preserve the viewer :)
		this.refresh();
	},
	
	refresh: function() {
		this.updateModel(function(updateAccessor) {
			if (this.__rootEntry) {
				this.__rootEntry._dispose();
			}
			this.__reverseItemMap= {};
			this.__rootEntry= null;
			var rebuildAccessor= this.__createRebuildAccessor();
			try {
				if (this.__modelTransformer != null && this.__input != null) {
					this.__modelTransformer.refreshModel(rebuildAccessor);
					this._resort();
				}
			} finally {
				rebuildAccessor.revokeAccess();
			}
			this.getSelectionProvider().setSelection([]);
		}, this);
	},
	
	getSorter: function() {
		return this.__sorter;
	},

	setSorter: function(sorter) {
		if (this.__sorter != null) {
			this.__sorter.inputChanged(null, null);
		}

		this.__sorter= sorter;

		if (this.__sorter != null) {
			this.__sorter.inputChanged(this, this.getInput());
		}
		this._resort();
	},
	
	refreshSortOrder: function() {
		this._resort();
	},

	addFilter: function(filter) {
		if (dojo.indexOf(this.__filters, filter) == -1) {
			this.__filters.push(filter);
		}
		this._refilter();
	},

	removeFilter: function(filter) {
		var filterIndex= dojo.indexOf(this.__filters, filter);
		if (filterIndex != -1) {
			this.__filters.splice(filterIndex, 1);
		}
		this._refilter();
	},

	refreshFilters: function() {
		this._refilter();
	},

	getViewer: function() {
		return this.__viewer;
	},

	addPostUpdateRunnable: function(code, runsBeforeFiltering) {
		var listName= this.__getPostUpdateListName(runsBeforeFiltering);
		this[listName].push(code);
		return {
			listName: listName,
			code: code
		};
	},

	removePostUpdateRunnable: function(token) {
		var runnableIndex= dojo.indexOf(this[token.listName], token.code);
		if (runnableIndex != -1) {
			this[token.listName].splice(runnableIndex, 1);
		}
	},

	readModel: function(code, thisObj) {
		var readAccessor= this.__createReadAccessor();
		try {
			return code.call(thisObj, readAccessor);
		} finally {
			readAccessor.revokeAccess();
		}
	},
	
	executeUnlocked: function(code, thisObj) {
		return this.updateModel(function(updateAccessor) {
			return updateAccessor.executeUnlocked(code, thisObj);
		}, this);
	},

	updateModel: function(code, thisObj) {
		var runnableResult;
		var viewerUpdates= null;

		this.__generationCounter++;
		var updateAccessor= this.__createUpdateAccessor();
		try {
			try {
				if (++this.__nestedUpdates == 1) {
					this.__doStartUpdate();
				}
				runnableResult= code.call(thisObj, updateAccessor);
			} finally {
				if (--this.__nestedUpdates == 0) {
					viewerUpdates= this.__doEndUpdate(updateAccessor);
				}
			}
		} finally {
			updateAccessor.revokeAccess();
		}

		if (viewerUpdates != null) {
			Assert.isNotNull(this.__viewer);
			for (var i= 0; i < viewerUpdates.length; i++) {
				viewerUpdates[i]();
			}
		}
		return runnableResult;
	},

	// ---- Selection API --------------------------------------------------------------------------------------------------

	getSelectionProvider: function() {
		return this.__selectionManager;
	},

	getSelectedEntries: function() {
		return this.__viewer.getSelectedEntries();
	},
	
	getSelectedElements: function() {
		var result= new Array();
		var entries=this.getSelectedEntries();
		for ( var i = 0; i < entries.length; i++) {
			result.push(entries[i].getElement());
		}
		return result;
	},
	
	setSelectedEntry: function(toSelect) {
		this.setEntrySelectionState(toSelect, true);
	},
	
	setSelectedEntries: function(toSelect) {
		// remove current selection
		dojo.forEach(this.getSelectedEntries(), function(entry) {
			this.setEntrySelectionState(entry, false);
		}, this);

		// select the new items
		for ( var i = 0; i < toSelect.length; i++) {
			this.__viewer.setSelectionState(toSelect[i], true);
		}

		// broadcast the selection
		this.getSelectionProvider().setSelection(this.getSelectedEntries());

	},
	
	setEntrySelectionState: function(entry, doSelect) {
		this.__viewer.setSelectionState(entry, doSelect);
		
		this.getSelectionProvider().setSelection(this.getSelectedEntries());
	},
	
	expandEntries: function(toExpand) {
		for ( var i = 0; i < toExpand.length; i++) {
			this.setEntryExpandState(toExpand[i], true);
		}
	},

	setEntryExpandState: function(entry, expanded) {
		this.__viewer.setExpandedState(entry, expanded);
	},

	revealEntry: function(toReveal) {
		this.__viewer.reveal(toReveal);
	},
	
	// XXX signature change
	getExpansionHull: function(hull, rootEntry) {
		// TODO
	},
	
	setExpansionHull: function(hull) {
		// TODO
	},

	// ---- ViewEntry's API ------------------------------------------------------------------------------------------------

	_getSorter: function() {
		return this.__sorter;
	},

	_calculateFilterState: function(entry, readAccessor) {
		var element= entry.getElement();
		if (element instanceof PlanItem && (element.isDirty() || element.isDraftItem())) {
			return true;
		}
		for (var i= 0; i < this.__filters.length; i++) {
			if (!this.__filters[i].select(entry, readAccessor))
				return false;
		}

		return true;
	},

	// ---- Viewer Update Implementation ----------------------------------------------------------------------------------- 

	__doStartUpdate: function() {
		if (this.__rootEntry != null) {
			this.__rootEntry._accept(function(entry) {
				return entry._doStartUpdate();
			}, false, this);
		}
		
		Assert.isTrue(this.__endUpdateRunnables == null);
		this.__endUpdateRunnables= [];
	},

	__doEndUpdate: function(updateAccessor) {
		var viewerUpdates= [];
		if (this.__rootEntry != null) {
			this.__runPostUpdateRunnables(updateAccessor, true);

			dojo.forEach(this.__rootEntry._getChildren(), function(child) {
				this.__filterRecursive(child, updateAccessor);
			}, this);

			this.__executePendingSortRequests(this.__rootEntry, updateAccessor);

			this.__runPostUpdateRunnables(updateAccessor, false);

			if (this.__viewer != null) {
				this.__rootEntry._accept(function(entry) {
					return entry._collectUpdates(this.__viewer, viewerUpdates);
				}, true, this);
			}
		} else if (this.__viewer != null) {
			viewerUpdates.push(com.ibm.team.apt.ui.model._ViewerUpdater.getRefreshRunnable(this.__viewer));
		}
		
		Assert.isNotNull(this.__endUpdateRunnables);
		if (this.__viewer != null) {
			viewerUpdates.push.apply(viewerUpdates, this.__endUpdateRunnables);
		}
		this.__endUpdateRunnables= null;
		return viewerUpdates;
	},

	__runPostUpdateRunnables: function(updateAccessor, isBeforeFiltering) {
		dojo.forEach(this[this.__getPostUpdateListName(isBeforeFiltering)], function(listener) {
			try {
				listener(updateAccessor);
			} catch (e) {
				console.group("postUpdateRunnable failed!"); //$NON-NLS-1$
				console.dir(e);
				console.groupEnd();
			}
		});
	},

	__getPostUpdateListName: function(beforeFiltering) {
		return beforeFiltering ? "__postUpdateBeforeFilterRunnables" : "__postUpdateAfterFilterRunnables"; //$NON-NLS-1$ //$NON-NLS-2$
	},

	__filterRecursive: function(entry, updateAccessor) {
		// TODO we filter ALL entries after each update
		var hasUnfilteredChildren= false;
		dojo.forEach(entry._getChildren(), function(child) {
			hasUnfilteredChildren|= !this.__filterRecursive(child, updateAccessor);
		}, this);

		var isFiltered= !this._calculateFilterState(entry, updateAccessor);
		if (isFiltered && !hasUnfilteredChildren && entry.isVisible()) {
			entry._hide(updateAccessor);
		} else if ((!isFiltered || hasUnfilteredChildren) && !entry.isVisible()) {
			entry._show(updateAccessor);
		}
		
		if (isFiltered && hasUnfilteredChildren) {
			if (!entry.hasTag(UnfilteredChildrenTag)) {
				updateAccessor.setTag(entry, UnfilteredChildrenTag);
			}
		} else if (entry.hasTag(UnfilteredChildrenTag)) {
			updateAccessor.clearTag(entry, UnfilteredChildrenTag);
		}
		
		return isFiltered && !hasUnfilteredChildren;
	},

	__executePendingSortRequests: function(entry, updateAccessor) {
		dojo.forEach(entry._getChildren(), function(child) {
			this.__executePendingSortRequests(child, updateAccessor);
		}, this);
		
		entry._doPendingSort(updateAccessor);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------

	_resort: function() {
		this.updateModel(function(updateAccessor) {
			if (this.__rootEntry != null) {
				this.__rootEntry._accept(function(entry) {
					entry._requestSortChildren(updateAccessor, true);
					return true;
				}, false, this);
			}
		}, this);
	},

	_refilter: function() {
		this.updateModel(function(updateAccessor) {
			/* intentionally empty -> filtering happens at the end of the update */
		}, this);
	},

	_doGetElementEntries: function(element) {
		var entries= this.__reverseItemMap[element.getElementKey()];

		var result= [];
		if (entries != null) {
			result.push.apply(result, entries);
		}
		return result;


	},

	_doAddEntry: function(path, newElement, updateAccessor) {
		if (this.__rootEntry == null) {
			throw new Error("illegal state"); //$NON-NLS-1$
		}

		// we can handle the leaf to be added very similar as the nodes
		path.push(newElement);
		try {
			var parent= this.__rootEntry;
			for (var i= 0; i < path.length; i++) {
				var currentEntry= this.__getExistingEntry(parent, path[i]);
				
				if (currentEntry === null) {
					currentEntry= this.__createEntry(path[i], parent, updateAccessor);
				}
	
				if (i < (path.length - 1)) {
					// this was just a node entry on the way to the leaf
					currentEntry._added(false);
				} else {
					// this is the leaf we wanted to add
					parent._requestSortChildren(updateAccessor, false);
					currentEntry._added(true);
				}
				
				parent= currentEntry;
			}
		} finally {
			// make sure we do not modify the input arg
			path.pop();
		}
		return parent;
	},

	_doRemoveEntry: function(entry, updateAccessor) {
		Assert.isLegal(entry != null);

		var toRemove= entry;
		while (toRemove != null && toRemove !== this.__rootEntry) {
			if (toRemove._removed(toRemove === entry)) {
				this.__unregisterEntry(toRemove, updateAccessor);
			}
			toRemove= toRemove._getParent();
		}
	},
	
	_doReparentEntry: function(entry, path, updateAccessor) {
		var parent= this.__rootEntry;
		for (var i= 0; i < path.length; i++) {
			parent= this.__getExistingEntry(parent, path[i]);
			Assert.isNotNull(parent);
		}

		return this.__doReparentEntryImpl(entry, parent, updateAccessor);
	},
	
	_doCreateRootEntry: function(element, updateAccessor) {
		Assert.isTrue(this.__rootEntry == null);
		 return this.__rootEntry= this.__createEntry(element, null, updateAccessor);
	},

	__doReparentEntryImpl: function(entry, parent, updateAccessor) {
		var i;
		var newEntry= entry._copy(parent);
		
		this.__registerEntry(newEntry.entry, updateAccessor);
		parent._add(newEntry.entry, updateAccessor);
		
		var candidate= newEntry.entry;
		while (candidate) {
			for (i= 0; i < newEntry.count; i++) {
				candidate._added(newEntry.entry == candidate);
			}
			candidate= candidate._getParent();
		}

		var children= entry._getChildren();
		for (i= 0; i < children.length; i++) {
			this.__doReparentEntryImpl(children[i], newEntry.entry, updateAccessor);
		}
		
		for (i= 0; i < newEntry.count; i++) {
			this._doRemoveEntry(entry, updateAccessor);
		}
		
		return newEntry.entry;
	},

	__getExistingEntry: function(parent, element) {
		var children= parent._getChildren();
		
		for (var i= 0; i < children.length; i++) {
			// FIXME this is potentially very slow
			if (element.equals(children[i].getElement())) {
				return children[i];
			}
		}
		return null;
	},

	__createEntry: function(element, parent, updateAccessor) {
		var childEntry= new ViewEntry(this, element, parent);
		this.__registerEntry(childEntry, updateAccessor);
		if (parent != null) {
			parent._add(childEntry, updateAccessor);
		}
		return childEntry;
	},


	__registerEntry: function(entry, updateAccessor) {
		var key= entry.getElement().getElementKey();

		var entrySet= this.__reverseItemMap[key];
		if (!entrySet) {
			entrySet= [];
			this.__reverseItemMap[key]= entrySet;
		}

		entrySet.push(entry);
	},

	__unregisterEntry: function(entry, updateAccessor) {
		var key= entry.getElement().getElementKey();
		var entrySet= this.__reverseItemMap[key];
		Assert.isNotNull(entrySet);
		
		if (entrySet.length > 1) {
			var entryIndex= dojo.indexOf(entrySet, entry);
			Assert.isTrue(entryIndex != -1);
			entrySet.splice(entryIndex, 1);
		} else {
			delete this.__reverseItemMap[key];
		}
	},

	// ---- Model accessor implementations ---------------------------------------------------------------------------------

	__createReadAccessor: function() {
		return new com.ibm.team.apt.ui.model.ViewModel._ModelReadAccessor(this);
	},
	
	__createUpdateAccessor: function() {
		return new com.ibm.team.apt.ui.model.ViewModel._ModelUpdateAccessor(this);
	},
	
	__createRebuildAccessor: function() {
		return new com.ibm.team.apt.ui.model.ViewModel._ModelRebuildAccessor(this);
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.model.ViewModel._LimitedModelAccessor", null, { //$NON-NLS-1$

	_viewModel: null,
	__isAccessGranted: true,
	
	constructor: function(viewModel) {
		this._viewModel= viewModel;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------
	
	revokeAccess: function() {
		this.__isAccessGranted= false;
	},

	getModel: function() {
		this._checkAccess();
		return this._viewModel;;
	},

	executeUnlocked: function(code) {
		this._checkAccess();

		this.__isAccessGranted= false;
		try {
			return code();
		} finally {
			this.__isAccessGranted= true;
		}
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_checkAccess: function(entry) {
		if (!this.__isAccessGranted) {
			throw new Error("The model access has been revoked"); //$NON-NLS-1$
		}
		
		if (entry && this._viewModel !== entry.getModel()) {
			throw new Error("ModelAccessor can't grant access to this ViewEntry"); //$NON-NLS-1$
		}
	},

	__sentinel: null // terminates this class definition
});

function invisibleEntryFilter(entry) {
	return entry.isVisible();
}

dojo.declare("com.ibm.team.apt.ui.model.ViewModel._ModelReadAccessor", //$NON-NLS-1$
			  com.ibm.team.apt.ui.model.ViewModel._LimitedModelAccessor, { 

	__visibleEntryNavigator: null,
	__allEntryNavigator: null,

	// ---- api ------------------------------------------------------------------------------------------------------------

	/**
	 * @param {Boolean} navigateVisibleOnly 
	 * @type com.ibm.team.apt.ui.model.EntryNavigator
	 */
	getEntryNavigator: function(navigateVisibleOnly) {
		this._checkAccess();
		
		var member= "__" + (navigateVisibleOnly ? "visible" : "all") + "EntryNavigator"; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
		if (!this[member]) {
			this[member]= new EntryNavigator(this, navigateVisibleOnly ? invisibleEntryFilter : null);
		}

		return this[member];
	},

	getFilterEntryNavigator: function(filterFn) {
		return new EntryNavigator(this, filterFn);
	},

	accept: function(visitor, startEntry, thisObj) {
		this._checkAccess();
		
		startEntry= startEntry || this._viewModel.__rootEntry; 
		if (startEntry) {
			startEntry._accept(visitor, false, thisObj);
		}
	},

	getRootEntry: function() {
		this._checkAccess();
		return this._viewModel.__rootEntry;
	},

	containsElement: function(element) {
		this._checkAccess();
		// FIXME we can't use elements as key!
		return this._viewModel.__reverseItemMap[element.getElementKey()];
	},

	getElementEntries: function(element) {
		this._checkAccess();
		return this._viewModel._doGetElementEntries(element);
	},

	getChildren: function(entry) {
		this._checkAccess(entry);
		return entry._getChildren();
	},

	getParent: function(entry) {
		this._checkAccess(entry);
		return entry._getParent();
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.model.ViewModel._ModelUpdateAccessor", //$NON-NLS-1$
			  com.ibm.team.apt.ui.model.ViewModel._ModelReadAccessor, { 

	// ---- api ------------------------------------------------------------------------------------------------------------

	addEntry: function(path, element) {
		this._checkAccess();
		return this._viewModel._doAddEntry(path, element, this);
	},
	
	reparentEntry: function(entry, path) {
		this._checkAccess();
		return this._viewModel._doReparentEntry(entry, path, this);
	},

	update: function(entry, properties) {
		this._checkAccess(entry);
		entry._update(properties, this);
	},

	setTag: function(entry, tag) {
		this._checkAccess(entry);
		entry._setTag(tag);
	},
	
	clearTag: function(entry, tag) {
		this._checkAccess(entry);
		entry._clearTag(tag);
	},

	move: function(entry, index) {
		this._checkAccess(entry);
		if (entry.isRootEntry()) {
			throw new Error("Can't move root item"); //$NON-NLS-1$
		}
		entry._getParent()._move(entry, index, this);
	},
	
	requestChildrenSort: function(entry, forceResort) {
		this._checkAccess(entry);
		entry._requestSortChildren(this, forceResort);
	},
	
	removeEntry: function(entry) {
		this._checkAccess(entry);
		this._viewModel._doRemoveEntry(entry, this);
	},
	
	isModified: function(entry) {
		return entry._hasUpdates();
	},
	
	executeAfterUpdate: function(code) {
		this._checkAccess();
		Assert.isNotNull(this._viewModel.__endUpdateRunnables);
		this._viewModel.__endUpdateRunnables.push(code);
	},

	__sentinel: null // terminates this class definition
});

dojo.declare("com.ibm.team.apt.ui.model.ViewModel._ModelRebuildAccessor", //$NON-NLS-1$ 
			  com.ibm.team.apt.ui.model.ViewModel._ModelUpdateAccessor, {

	// ---- api ------------------------------------------------------------------------------------------------------------

	createRootEntry: function(element) {
		this._checkAccess();
		return this._viewModel._doCreateRootEntry(element, this);
	},

	__sentinel: null // terminates this class definition
});

})();
